@@ -238,18 +238,17 @@ void annotatemore_init(int myflags,
     init_annotation_definitions();
 }
 
-void annotatemore_open(const char *fname)
+void annotatemore_open(char *fname)
 {
     int ret;
     char *tofree = NULL;
 
-    if (!fname)
-	fname = config_getstring(IMAPOPT_ANNOTATION_DB_PATH);
-
     /* create db file name */
     if (!fname) {
-	tofree = strconcat(config_dir, FNAME_ANNOTATIONS, (char *)NULL);
-	fname = tofree;
+	fname = xmalloc(strlen(config_dir)+sizeof(FNAME_ANNOTATIONS));
+	tofree = fname;
+	strcpy(fname, config_dir);
+	strcat(fname, FNAME_ANNOTATIONS);
     }
 
     ret = (DB->open)(fname, CYRUSDB_CREATE, &anndb);
@@ -259,7 +258,7 @@ void annotatemore_open(const char *fname)
 	fatal("can't read annotations file", EC_TEMPFAIL);
     }    
 
-    free(tofree);
+    if (tofree) free(tofree);
 
     annotate_dbopen = 1;
 }
@@ -1428,14 +1427,6 @@ int annotatemore_write_entry(const char *mboxname, const char *entry,
     return write_entry(mboxname, entry, userid, &theentry, tid);
 }
 
-int annotatemore_commit(struct txn *tid) {
-    return tid ? DB->commit(anndb, tid) : 0;
-}
-
-int annotatemore_abort(struct txn *tid) {
-    return tid ? DB->abort(anndb, tid) : 0;
-}
-
 struct storedata {
     struct namespace *namespace;
     const char *userid;
@@ -1726,7 +1717,7 @@ static int annotation_set_mailboxopt(const char *int_mboxname,
 {
     struct mailbox *mailbox = NULL;
     int flag = 0, r = 0, i;
-    unsigned long newopts;
+    unsigned long oldopts;
 
     /* Check entry */
     for (i = 0; annotate_mailbox_flags[i].name; i++) {
@@ -1748,23 +1739,22 @@ static int annotation_set_mailboxopt(const char *int_mboxname,
     r = mailbox_open_iwl(int_mboxname, &mailbox);
     if (r) return r;
 
-    newopts = mailbox->i.options;
+    oldopts = mailbox->i.options;
 
     if (!strcmp(entry->shared.value, "true")) {
-	newopts |= flag;
+	mailbox->i.options |= flag;
     } else {
-	newopts &= ~flag;
+	mailbox->i.options &= ~flag;
     }
 
-    /* only commit if there's been a change */
-    if (mailbox->i.options != newopts) {
+    if (oldopts != mailbox->i.options) {
 	mailbox_index_dirty(mailbox);
-	mailbox->i.options = newopts;
+	r = mailbox_commit(mailbox);
     }
 
     mailbox_close(&mailbox);
 
-    return 0;
+    return r;
 }
 
 const struct annotate_st_entry server_entries[] =
@@ -2283,8 +2273,7 @@ void init_annotation_definitions()
 	ae = xmalloc(sizeof(struct annotate_st_entry));
 
 	p2 = p;
-	for (; *p && (isalnum(*p) ||
-		      *p=='.' || *p=='-' || *p=='_' || *p=='/' || *p==':');
+	for (; *p && (isalnum(*p) || *p=='.' || *p=='-' || *p=='_' || *p=='/');
 	     p++);
 	/* TV-TODO: should test for empty */
 	ae->name = xstrndup(p2, p-p2);
