@@ -242,8 +242,7 @@ int append_commit(struct appendstate *as,
 	append_addseen(as->mailbox, as->userid, as->seen_seq);
     seqset_free(as->seen_seq);
     
-    /* We want to commit here to guarantee mailbox on disk vs
-     * duplicate DB consistency */
+    /* Write out index header & synchronize to disk. */
     r = mailbox_commit(as->mailbox);
     if (r) {
 	syslog(LOG_ERR, "IOERROR: commiting mailbox append %s: %s",
@@ -351,7 +350,7 @@ FILE *append_newstage(const char *mailboxname, time_t internaldate,
  */
 int append_fromstage(struct appendstate *as, struct body **body,
 		     struct stagemsg *stage, time_t internaldate,
-		     const strarray_t *flags, int nolink)
+		     const char **flag, int nflags, int nolink)
 {
     struct mailbox *mailbox = as->mailbox;
     struct index_record message_index;
@@ -456,7 +455,7 @@ int append_fromstage(struct appendstate *as, struct body **body,
     if (destfile) {
 	/* this will hopefully ensure that the link() actually happened
 	   and makes sure that the file actually hits disk */
-	fsync(fileno(destfile));
+	r = fsync(fileno(destfile));
 	fclose(destfile);
     }
     if (r) {
@@ -465,34 +464,33 @@ int append_fromstage(struct appendstate *as, struct body **body,
     }
 
     /* Handle flags the user wants to set in the message */
-    for (i = 0; flags && i < flags->count ; i++) {
-	const char *flag = strarray_nth(flags, i);
-	if (!strcmp(flag, "\\seen")) {
+    for (i = 0; i < nflags; i++) {
+	if (!strcmp(flag[i], "\\seen")) {
 	    append_setseen(as, &message_index);
 	}
-	else if (!strcmp(flag, "\\deleted")) {
+	else if (!strcmp(flag[i], "\\deleted")) {
 	    if (as->myrights & ACL_DELETEMSG) {
 		message_index.system_flags |= FLAG_DELETED;
 	    }
 	}
-	else if (!strcmp(flag, "\\draft")) {
+	else if (!strcmp(flag[i], "\\draft")) {
 	    if (as->myrights & ACL_WRITE) {
 		message_index.system_flags |= FLAG_DRAFT;
 	    }
 	}
-	else if (!strcmp(flag, "\\flagged")) {
+	else if (!strcmp(flag[i], "\\flagged")) {
 	    if (as->myrights & ACL_WRITE) {
 		message_index.system_flags |= FLAG_FLAGGED;
 	    }
 	}
-	else if (!strcmp(flag, "\\answered")) {
+	else if (!strcmp(flag[i], "\\answered")) {
 	    if (as->myrights & ACL_WRITE) {
 		message_index.system_flags |= FLAG_ANSWERED;
 	    }
 	}
 	else if (as->myrights & ACL_WRITE) {
 	    /* User flag */
-	    r = mailbox_user_flag(mailbox, flag, &userflag);
+	    r = mailbox_user_flag(mailbox, flag[i], &userflag);
 	    if (!r) 
 		message_index.user_flags[userflag/32] |= 1<<(userflag&31);
 	}
@@ -734,22 +732,22 @@ static int append_addseen(struct mailbox *mailbox,
 			  struct seqset *newseen)
 {
     int r;
-    struct seen *seendb = NULL;
-    struct seendata sd = SEENDATA_INITIALIZER;
+    struct seen *seendb;
+    struct seendata sd;
     struct seqset *oldseen;
 
     if (!newseen->len)
 	return 0;
 
     r = seen_open(userid, SEEN_CREATE, &seendb);
-    if (r) goto done;
+    if (r) return r;
 
     r = seen_lockread(seendb, mailbox->uniqueid, &sd);
     if (r) goto done;
 
     /* parse the old sequence */
     oldseen = seqset_parse(sd.seenuids, NULL, mailbox->i.last_uid);
-    seen_freedata(&sd);
+    free(sd.seenuids);
 
     /* add the extra items */
     seqset_join(oldseen, newseen);
@@ -759,9 +757,9 @@ static int append_addseen(struct mailbox *mailbox,
     /* and write it out */
     sd.lastchange = time(NULL);
     r = seen_write(seendb, mailbox->uniqueid, &sd);
-    seen_freedata(&sd);
+    free(sd.seenuids);
 
  done:
-    seen_close(&seendb);
+    seen_close(seendb);
     return r;
 }
