@@ -74,7 +74,6 @@
 #include <sasl/sasl.h>
 
 #include "assert.h"
-#include "annotate.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "global.h"
@@ -1067,17 +1066,11 @@ int main(int argc, char *argv[])
 	mboxlist_init(0);
 	mboxlist_open(mboxdb_fname);
 
-	quotadb_init(0);
-	quotadb_open(NULL);
-
-	annotatemore_init(0, NULL, NULL);
-	annotatemore_open(NULL);
-
+ 	quotadb_init(0);
+ 	quotadb_open(NULL);
+	
 	do_dump(op, partition, dopurge);
 
-	annotatemore_close();
-	annotatemore_done();
-
 	quotadb_close();
 	quotadb_done();
 	
@@ -1089,17 +1082,11 @@ int main(int argc, char *argv[])
 	mboxlist_init(0);
 	mboxlist_open(mboxdb_fname);
 
-	quotadb_init(0);
-	quotadb_open(NULL);
-
-	annotatemore_init(0, NULL, NULL);
-	annotatemore_open(NULL);
+ 	quotadb_init(0);
+ 	quotadb_open(NULL);
 
 	do_undump();
 
-	annotatemore_close();
-	annotatemore_done();
-
 	quotadb_close();
 	quotadb_done();
 	
