@@ -53,8 +53,6 @@
 #include "signals.h"
 #include "util.h"
 
-#define MAX_SESSIONID_SIZE 256
-
 /* Flags for cyrus_init() */
 enum {
     CYRUSINIT_NODB =	(1<<0)
@@ -80,8 +78,6 @@ extern int mysasl_config(void *context,
 			 unsigned *len);
 extern sasl_security_properties_t *mysasl_secprops(int flags);
 
-typedef int (mysasl_cb_ft)(void);
-
 /* user canonification */
 extern char *canonify_userid(char *user, char *loginid, int *domain_from_ip);
 
@@ -166,9 +162,5 @@ extern struct cyrusdb_backend *config_userdeny_db;
 /* Session ID */
 extern void session_new_id();
 extern const char *session_id();
-extern void parse_sessionid(const char *str, char *sessionid);
-
-/* Capability suppression */
-extern int capa_is_disabled(const char *str);
 
 #endif /* INCLUDED_GLOBAL_H */
