@@ -38,6 +38,8 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * $Id: lmtpd.c,v 1.167 2010/05/25 20:59:19 wescraig Exp $
  */
 
 #include <config.h>
@@ -88,7 +90,6 @@
 #include "statuscache.h"
 #include "telemetry.h"
 #include "tls.h"
-#include "userdeny.h"
 #include "util.h"
 #include "version.h"
 #include "xmalloc.h"
@@ -163,9 +164,9 @@ static struct protocol_t lmtp_protocol =
 };
 
 static struct sasl_callback mysasl_cb[] = {
-    { SASL_CB_GETOPT, (mysasl_cb_ft *) &mysasl_config, NULL },
-    { SASL_CB_PROXY_POLICY, (mysasl_cb_ft *) &mysasl_proxy_policy, NULL },
-    { SASL_CB_CANON_USER, (mysasl_cb_ft *) &mysasl_canon_user, NULL },
+    { SASL_CB_GETOPT, &mysasl_config, NULL },
+    { SASL_CB_PROXY_POLICY, &mysasl_proxy_policy, NULL },
+    { SASL_CB_CANON_USER, &mysasl_canon_user, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
@@ -225,10 +226,6 @@ int service_init(int argc __attribute__((unused)),
 	quotadb_init(0);
 	quotadb_open(NULL);
 
-	/* open the user deny db */
-	denydb_init(0);
-	denydb_open(NULL);
-
 	/* Initialize the annotatemore db (for sieve on shared mailboxes) */
 	annotatemore_init(0, NULL, NULL);
 	annotatemore_open(NULL);
@@ -270,6 +267,8 @@ int service_main(int argc, char **argv,
 {
     int opt, r;
 
+    session_new_id();
+
     sync_log_init();
 
     deliver_in = prot_new(0, 0);
@@ -339,7 +338,8 @@ void service_abort(int error)
     shut_down(error);
 }
 
-static void usage()
+static void
+usage()
 {
     fprintf(stderr, "421-4.3.0 usage: lmtpd [-C <alt_config>] [-a]\r\n");
     fprintf(stderr, "421 4.3.0 %s\n", cyrus_version());
@@ -448,8 +448,6 @@ static int mlookup(const char *name, char **server, char **aclp, void *tid)
 	    fatal("error communicating with MUPDATE server", EC_TEMPFAIL);
 	}
 
-	if (mailboxdata->t == RESERVE) return IMAP_MAILBOX_RESERVED;
-
 	if (aclp) *aclp = (char *) mailboxdata->acl;
 	if (server) *server = (char *) mailboxdata->server;
 	c = strchr(*server, '!');
@@ -464,8 +462,6 @@ static int mlookup(const char *name, char **server, char **aclp, void *tid)
 	    r = mboxlist_lookup(name, &mbentry, tid);
 	}
 	if (r) return r;
-	if (mbentry.mbtype & MBTYPE_MOVING)  return IMAP_MAILBOX_MOVED;
-	if (mbentry.mbtype & MBTYPE_DELETED) return IMAP_MAILBOX_NONEXISTENT;
 
 	if (aclp) *aclp = mbentry.acl;
 	if (server) {
@@ -481,7 +477,7 @@ static int mlookup(const char *name, char **server, char **aclp, void *tid)
     return r;
 }
 
-/* places msg in mailbox mailboxname.
+/* places msg in mailbox mailboxname.  
  * if you wish to use single instance store, pass stage as non-NULL
  * if you want to deliver message regardless of duplicates, pass id as NULL
  * if you want to notify, pass user
@@ -492,22 +488,22 @@ int deliver_mailbox(FILE *f,
 		    struct message_content *content,
 		    struct stagemsg *stage,
 		    unsigned size,
-		    const strarray_t *flags,
+		    char **flag,
+		    int nflags,
 		    char *authuser,
 		    struct auth_state *authstate,
 		    char *id,
 		    const char *user,
 		    char *notifyheader,
 		    const char *mailboxname,
-		    char *date,
 		    int quotaoverride,
 		    int acloverride)
 {
     int r;
     struct appendstate as;
+    time_t now = time(NULL);
     unsigned long uid;
     const char *notifier;
-    duplicate_key_t dkey = {NULL, NULL, NULL};
 
     r = append_setup(&as, mailboxname,
 		     authuser, authstate, acloverride ? 0 : ACL_POST, 
@@ -516,12 +512,9 @@ int deliver_mailbox(FILE *f,
 		     (long) size : 0);
 
     /* check for duplicate message */
-    dkey.id = id;
-    dkey.to = mailboxname;
-    dkey.date = date;
     if (!r && id && dupelim && !(as.mailbox->i.options & OPT_IMAP_DUPDELIVER) &&
-	duplicate_check(&dkey)) {
-	duplicate_log(&dkey, "delivery");
+	duplicate_check(id, strlen(id), mailboxname, strlen(mailboxname))) {
+	duplicate_log(id, mailboxname, "delivery");
 	append_abort(&as);
 	return 0;
     }
@@ -533,21 +526,21 @@ int deliver_mailbox(FILE *f,
     }
 
     if (!r) {
-	r = append_fromstage(&as, &content->body, stage, 0,
-			     flags, !singleinstance);
+	r = append_fromstage(&as, &content->body, stage, now,
+			     (const char **) flag, nflags, !singleinstance);
 
 	if (r) {
 	    append_abort(&as);
 	} else {
 	    struct mailbox *mailbox = NULL;
-	    /* hold the mailbox open until the duplicate mark is done */
 	    r = append_commit(&as, quotaoverride ? -1 : 0, NULL, &uid,
 			      NULL, &mailbox);
 	    if (!r) {
 		syslog(LOG_INFO, "Delivered: %s to mailbox: %s",
 		       id, mailboxname);
 		if (dupelim && id) {
-		    duplicate_mark(&dkey, time(NULL), uid);
+		    duplicate_mark(id, strlen(id), mailboxname, 
+				   strlen(mailboxname), now, uid);
 		}
 		mailbox_close(&mailbox);
 	    }
@@ -568,7 +561,7 @@ int deliver_mailbox(FILE *f,
 	    if (strlen(mailboxname) >= inboxlen &&
 		!strncmp(mailboxname, inbox, inboxlen) &&
 		(!mailboxname[inboxlen] || mailboxname[inboxlen] == '.')) {
-		strlcpy(inbox, "INBOX", sizeof(inbox));
+		strlcpy(inbox, "INBOX", sizeof(inbox)); 
 		strlcat(inbox, mailboxname+inboxlen, sizeof(inbox));
 		notify_mailbox = inbox;
 	    }
@@ -612,6 +605,7 @@ void deliver_remote(message_data_t *msgdata,
 	struct rcpt *rc;
 	struct backend *remote;
 	int i = 0;
+	int r = 0;
 	
 	lt->from = msgdata->return_path;
 	lt->auth = d->authas[0] ? d->authas : NULL;
@@ -633,10 +627,7 @@ void deliver_remote(message_data_t *msgdata,
 	remote = proxy_findserver(d->server, &lmtp_protocol, "",
 				  &backend_cached, NULL, NULL, NULL);
 	if (remote) {
-	    int txn_timeout = config_getint(IMAPOPT_LMTPTXN_TIMEOUT);
-	    if (txn_timeout) 
-		prot_settimeout(remote->in, txn_timeout);
-	    lmtp_runtxn(remote, lt);
+	    r = lmtp_runtxn(remote, lt);
 	} else {
 	    /* remote server not available; tempfail all deliveries */
 	    for (rc = d->to, i = 0; i < d->rnum; i++) {
@@ -651,7 +642,7 @@ void deliver_remote(message_data_t *msgdata,
 	    int j = rc->rcpt_num;
 	    switch (status[j]) {
 	    case s_wait:
-		/* hmmm, if something fails we'll want to try an
+		/* hmmm, if something fails we'll want to try an 
 		   error delivery */
 		if (lt->rcpt[i].result != RCPT_GOOD) {
 		    status[j] = s_err;
@@ -680,7 +671,7 @@ void deliver_remote(message_data_t *msgdata,
     }
 }
 
-int deliver_local(deliver_data_t *mydata, const strarray_t *flags,
+int deliver_local(deliver_data_t *mydata, char **flag, int nflags,
 		  const char *username, const char *mailboxname)
 {
     char namebuf[MAX_MAILBOX_BUFFER] = "", *tail;
@@ -694,10 +685,10 @@ int deliver_local(deliver_data_t *mydata, const strarray_t *flags,
 	strlcat(namebuf, mailboxname, sizeof(namebuf));
 
 	return deliver_mailbox(md->f, mydata->content, mydata->stage,
-			       md->size, flags,
+			       md->size, flag, nflags,
 			       mydata->authuser, mydata->authstate, md->id,
 			       NULL, mydata->notifyheader,
-			       namebuf, md->date, quotaoverride, 0);
+			       namebuf, quotaoverride, 0);
     }
 
     /* case 2: ordinary user */
@@ -714,20 +705,20 @@ int deliver_local(deliver_data_t *mydata, const strarray_t *flags,
 	    strlcat(namebuf, mailboxname, sizeof(namebuf));
 
 	    ret2 = deliver_mailbox(md->f, mydata->content, mydata->stage,
-				   md->size, flags,
+				   md->size, flag, nflags,
 				   mydata->authuser, mydata->authstate, md->id,
 				   username, mydata->notifyheader,
-				   namebuf, md->date, quotaoverride, 0);
+				   namebuf, quotaoverride, 0);
 	}
 	if (ret2 == IMAP_MAILBOX_NONEXISTENT && mailboxname &&
 	    config_getswitch(IMAPOPT_LMTP_FUZZY_MAILBOX_MATCH) &&
 	    fuzzy_match(namebuf)) {
 	    /* try delivery to a fuzzy matched mailbox */
 	    ret2 = deliver_mailbox(md->f, mydata->content, mydata->stage,
-				   md->size, flags,
+				   md->size, flag, nflags,
 				   mydata->authuser, mydata->authstate, md->id,
 				   username, mydata->notifyheader,
-				   namebuf, md->date, quotaoverride, 0);
+				   namebuf, quotaoverride, 0);
 	}
 	if (ret2) {
 	    /* normal delivery to INBOX */
@@ -736,10 +727,10 @@ int deliver_local(deliver_data_t *mydata, const strarray_t *flags,
 	    *tail = '\0';
 
 	    ret = deliver_mailbox(md->f, mydata->content, mydata->stage,
-				  md->size, flags,
+				  md->size, flag, nflags,
 				  (char *) username, authstate, md->id,
 				  username, mydata->notifyheader,
-				  namebuf, md->date, quotaoverride, 1);
+				  namebuf, quotaoverride, 1);
 
 	    if (authstate) auth_freestate(authstate);
 	}
@@ -757,7 +748,7 @@ int deliver(message_data_t *msgdata, char *authuser,
     struct message_content content = { NULL, 0, NULL };
     char *notifyheader;
     deliver_data_t mydata;
-
+    
     assert(msgdata);
     nrcpts = msg_getnumrcpt(msgdata);
     assert(nrcpts);
@@ -775,7 +766,7 @@ int deliver(message_data_t *msgdata, char *authuser,
     mydata.namespace = &lmtpd_namespace;
     mydata.authuser = authuser;
     mydata.authstate = authstate;
-
+    
     /* loop through each recipient, attempting delivery for each */
     for (n = 0; n < nrcpts; n++) {
 	char namebuf[MAX_MAILBOX_BUFFER] = "", *server;
@@ -825,7 +816,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 #endif
 
 	    if (r) {
-		r = deliver_local(&mydata, NULL, userbuf, mailbox);
+		r = deliver_local(&mydata, NULL, 0, userbuf, mailbox);
 	    }
 	}
 
@@ -844,7 +835,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 	while (d) {
 	    struct dest *nextd = d->next;
 	    struct rcpt *rc = d->to;
-
+   
 	    while (rc) {
 		struct rcpt *nextrc = rc->next;
 		free(rc);
@@ -862,7 +853,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 	    case s_err:
 	    case s_done:
 		/* yikes, we haven't implemented sieve ! */
-		syslog(LOG_CRIT,
+		syslog(LOG_CRIT, 
 		       "sieve states reached, but we don't implement sieve");
 		abort();
 	    break;
@@ -886,7 +877,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 	    assert(status[n] == done || status[n] == s_done);
 	}
     }
-
+   
     /* cleanup */
     free(status);
     if (content.base) map_free(&content.base, &content.len);
@@ -904,7 +895,7 @@ int deliver(message_data_t *msgdata, char *authuser,
 void fatal(const char* s, int code)
 {
     static int recurse_code = 0;
-
+    
     if(recurse_code) {
 	/* We were called recursively. Just give up */
 	snmp_increment(ACTIVE_CONNECTIONS, -1);
@@ -918,7 +909,7 @@ void fatal(const char* s, int code)
     if (stage) append_removestage(stage);
 
     syslog(LOG_ERR, "FATAL: %s", s);
-
+    
     /* shouldn't return */
     shut_down(code);
 
@@ -961,9 +952,6 @@ void shut_down(int code)
 	quotadb_close();
 	quotadb_done();
 
-	denydb_close();
-	denydb_done();
-
 	annotatemore_close();
 	annotatemore_done();
 
@@ -1060,18 +1048,6 @@ static int verify_user(const char *user, const char *domain, char *mailbox,
 	}
     }
 
-    if (!r && user) {
-	char msg[MAX_MAILBOX_PATH+1];
-
-	if (domain) {
-	    snprintf(namebuf, sizeof(namebuf), "%s@%s", user, domain);
-	    user = namebuf;
-	}
-
-	if (userdeny(user, config_ident, msg, sizeof(msg)))
-	    return IMAP_MAILBOX_DISABLED;
-    }
-
     if (r) syslog(LOG_DEBUG, "verify_user(%s) failed: %s", namebuf,
 		  error_message(r));
 
@@ -1102,7 +1078,7 @@ char *generate_notify(message_data_t *m)
 		    ret = xrealloc(ret, len += 1024);
 		}
 		pos += sprintf(ret + pos, "%s: ", h);
-
+		
 		/* put the header body.
 		   xxx it would be nice to linewrap.*/
 		/* need: length + '\n' + '\0' */
