@@ -38,6 +38,8 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * $Id: lmtpengine.c,v 1.132 2010/01/06 17:01:35 murch Exp $
  */
 
 #include <config.h>
@@ -220,15 +222,6 @@ static void send_lmtp_error(struct protstream *pout, int r)
 	prot_printf(pout, "451 4.2.1 Mailbox Moved\r\n");
 	break;
 
-    case IMAP_MAILBOX_RESERVED:
-	prot_printf(pout, "451 4.2.1 Mailbox Reserved\r\n");
-	break;
-
-    case IMAP_MAILBOX_DISABLED:
-	prot_printf(pout,
-		    "450 4.2.1 Mailbox disabled, not accepting messages\r\n");
-	break;
-
     case IMAP_MESSAGE_CONTAINSNULL:
 	prot_printf(pout, "554 5.6.0 Message contains NUL characters\r\n");
 	break;
@@ -289,7 +282,6 @@ int msg_new(message_data_t **m)
     ret->return_path = NULL;
     ret->rcpt = NULL;
     ret->rcpt_num = 0;
-    ret->date = NULL;
 
     ret->authuser = NULL;
     ret->authstate = NULL;
@@ -327,9 +319,6 @@ void msg_free(message_data_t *m)
 	}
 	free(m->rcpt);
     }
-    if (m->date) {
-      free(m->date);
-     }
 
     if (m->authuser) {
 	free(m->authuser);
@@ -473,7 +462,6 @@ static char *parseaddr(char *s)
 {
     char *p, *ret;
     int len;
-    int lmtp_strict_rfc2821 = config_getswitch(IMAPOPT_LMTP_STRICT_RFC2821);
 
     p = s;
 
@@ -512,12 +500,6 @@ static char *parseaddr(char *s)
 		if (!*++p) return 0;
 	    }
 	    else {
-		if (*p & 128 && !lmtp_strict_rfc2821) {
-		    /* this prevents us from becoming a backscatter
-		       source if our MTA allows 8bit in local-part
-		       of adresses. */
-		    *p = 'X';
-		}
 		if (*p <= ' ' || (*p & 128) ||
 		    strchr("<>()[]\\,;:\"", *p)) return 0;
 	    }
@@ -537,7 +519,7 @@ static char *parseaddr(char *s)
 	    while (Uisalnum(*p) || *p == '.' || *p == '-') p++;
 	}
     }
-
+    
     if (*p++ != '>') return 0;
     if (*p && *p != ' ') return 0;
     len = p - s;
@@ -549,7 +531,7 @@ static char *parseaddr(char *s)
 }
 
 /* clean off the <> from the return path */
-static void clean_retpath(char *rpath)
+void clean_retpath(char *rpath)
 {
     int sl;
 
@@ -570,7 +552,9 @@ static void clean_retpath(char *rpath)
  * from string pointed to by 'buf'.  Does not handle continuation header
  * lines.
  */
-static void clean822space(char* buf)
+void
+clean822space(buf)
+char *buf;
 {
     char *from=buf, *to=buf;
     int c;
@@ -750,16 +734,12 @@ static int savemsg(struct clientdata *cd,
     }
 
     /* get date */
-    if (!(body = spool_getheader(m->hdrcache, "date"))) {
+    if (!spool_getheader(m->hdrcache, "date")) {
 	/* no date, create one */
 	addbody = xstrdup(datestr);
-	m->date = xstrdup(datestr);
 	fprintf(f, "Date: %s\r\n", addbody);
 	spool_cache_header(xstrdup("Date"), addbody, m->hdrcache);
     }
-    else {
-	m->date = xstrdup(body[0]);
-    }
 
     if (!m->return_path &&
 	(body = msg_getheader(m, "return-path"))) {
@@ -948,7 +928,7 @@ static int localauth_mechlist_override(
 }
 
 static struct sasl_callback localauth_override_cb[] = {
-    { SASL_CB_GETOPT, (mysasl_cb_ft *) &localauth_mechlist_override, NULL },
+    { SASL_CB_GETOPT, &localauth_mechlist_override, NULL },
     { SASL_CB_LIST_END, NULL, NULL },
 };
 
@@ -1238,7 +1218,7 @@ void lmtpmode(struct lmtp_func *func,
 				  strlcpy(hbuf, "[unknown]", sizeof(hbuf));
 			  }
 			  else
-			      strlcpy(hbuf, "[unix socket]", sizeof(hbuf));
+			      strlcpy(hbuf, "[unix socket]", sizeof(hbuf));		  
 			  syslog(LOG_ERR, "badlogin: %s %s %s",
 				 hbuf, mech, sasl_errdetail(cd.conn));
 		  
@@ -1351,10 +1331,10 @@ void lmtpmode(struct lmtp_func *func,
 	      else
 		  prot_printf(pout, "250-SIZE\r\n");
 	      if (tls_enabled() && !cd.starttls_done &&
-		  cd.authenticated == NOAUTH) {
+		  cd.authenticated <= NOAUTH) {
 		  prot_printf(pout, "250-STARTTLS\r\n");
 	      }
-	      if ((cd.authenticated <= 0) &&
+	      if ((cd.authenticated <= NOAUTH || saslprops.ssf) &&
 		  sasl_listmech(cd.conn, NULL, "AUTH ", " ", "", &mechs, 
 				NULL, &mechcount) == SASL_OK && 
 		  mechcount > 0) {
@@ -1364,7 +1344,6 @@ void lmtpmode(struct lmtp_func *func,
 
 	      strlcpy(cd.lhlo_param, buf + 5, sizeof(cd.lhlo_param));
 	      
-	      session_new_id();
 	      continue;
 	  }
 	  goto syntaxerr;
@@ -1559,8 +1538,7 @@ void lmtpmode(struct lmtp_func *func,
 		continue;
 	    }
 	    else if (!strcasecmp(buf, "rset")) {
-		session_new_id();
-		prot_printf(pout, "250 2.0.0 ok SESSIONID=<%s>\r\n", session_id());
+		prot_printf(pout, "250 2.0.0 ok\r\n");
 
 	      rset:
 		if (msg) msg_free(msg);
@@ -1579,9 +1557,6 @@ void lmtpmode(struct lmtp_func *func,
 		sasl_ssf_t ssf;
 		char *auth_id;
 
-		/* XXX  discard any input pipelined after STARTTLS */
-		prot_flush(pin);
-
 		/* SASL and openssl have different ideas
 		   about whether ssf is signed */
 		layerp = (int *) &ssf;
@@ -1719,13 +1694,6 @@ static int revconvert_lmtp(const char *code)
 	else if (code[4] == '4' && code [6] == '4') {
 	    return IMAP_SERVER_UNAVAILABLE;
 	}
-	else if (code[4] == '4' && code[6] == '2') {
-	    if (code[8] == '1') {
-		return IMAP_MAILBOX_MOVED;
-	    } else {
-		return IMAP_MAILBOX_BADFORMAT;
-	    }
-	}
 	else {
 	    return IMAP_IOERROR;
 	}
@@ -1849,17 +1817,14 @@ static void pushmsg(struct protstream *in, struct protstream *out,
 
     if (!isdotstuffed) {
 	/* signify end of message */
-	if (!lastline_hadendline) {
-	    prot_printf(out, "\r\n");
-	}
-	prot_printf(out, ".\r\n");
+	prot_printf(out, "\r\n.\r\n");
     }
 }
 
 int lmtp_runtxn(struct backend *conn, struct lmtp_txn *txn)
 {
     int j, code, r = 0;
-    char buf[8192], rsessionid[MAX_SESSIONID_SIZE];
+    char buf[8192];
     int onegood;
 
     assert(conn && txn);
@@ -1874,11 +1839,6 @@ int lmtp_runtxn(struct backend *conn, struct lmtp_txn *txn)
 	goto failall;
     }
 
-    if (config_auditlog) {
-	parse_sessionid(buf, rsessionid);
-	syslog(LOG_NOTICE, "auditlog: proxy sessionid=<%s> remote=<%s>", session_id(), rsessionid);
-    }
-
     /* mail from */
     if (!txn->from) {
 	prot_printf(conn->out, "MAIL FROM:<>");
