@@ -52,6 +52,7 @@
 
 #include "auth.h"
 #include "byteorder64.h"
+#include "mboxlist.h"
 #include "message_guid.h"
 #include "prot.h"
 #include "quota.h"
@@ -457,15 +458,13 @@ extern int mailbox_rewrite_index_record(struct mailbox *mailbox,
 				        struct index_record *record);
 extern int mailbox_append_index_record(struct mailbox *mailbox,
 				       struct index_record *record);
-extern int mailbox_find_index_record(struct mailbox *mailbox, uint32_t uid,
-				     struct index_record *record);
 
-extern int mailbox_set_acl(struct mailbox *mailbox, const char *acl,
-			   int dirty_modseq);
+extern int mailbox_set_acl(struct mailbox *mailbox, const char *acl);
 extern int mailbox_set_quotaroot(struct mailbox *mailbox, const char *quotaroot);
 extern int mailbox_user_flag(struct mailbox *mailbox, const char *flag,
 			     int *flagnum);
 extern int mailbox_commit(struct mailbox *mailbox);
+extern int mailbox_commit_cache(struct mailbox *mailbox);
 
 /* seen state check */
 extern int mailbox_internal_seen(struct mailbox *mailbox, const char *userid);
@@ -486,17 +485,12 @@ extern int mailbox_create(const char *name, const char *part, const char *acl,
 			  const char *uniqueid, int options, unsigned uidvalidity,
 			  struct mailbox **mailboxptr);
 
-extern int mailbox_copy_files(struct mailbox *mailbox, const char *newpart,
-			      const char *newname);
-extern int mailbox_delete_cleanup(const char *part, const char *name);
-
 extern int mailbox_rename_copy(struct mailbox *oldmailbox, 
 			       const char *newname, const char *newpart,
 			       const char *userid, int ignorequota,
 			       struct mailbox **newmailboxptr);
 extern int mailbox_rename_cleanup(struct mailbox **mailboxptr, int isinbox);
 
-
 extern int mailbox_copyfile(const char *from, const char *to, int nolink);
 
 extern int mailbox_reconstruct(const char *name, int flags);
