@@ -86,8 +86,7 @@
 
 static int dump_file(int first, int sync,
 		     struct protstream *pin, struct protstream *pout,
-		     const char *filename, const char *ftag,
-		     const char *fbase, unsigned long flen);
+		     const char *filename, const char *ftag);
 
 /* support for downgrading index files on copying back to an
  * earlier version of Cyrus */
@@ -263,7 +262,7 @@ static int dump_index(struct mailbox *mailbox, int oldversion,
     }
 
     close(oldindex_fd);
-    r = dump_file(first, sync, pin, pout, oldname, "cyrus.index", NULL, 0);
+    r = dump_file(first, sync, pin, pout, oldname, "cyrus.index");
     unlink(oldname);
     if (r) return r;
 
@@ -294,7 +293,7 @@ static int dump_index(struct mailbox *mailbox, int oldversion,
 	}
 
 	close(oldindex_fd);
-	r = dump_file(first, sync, pin, pout, oldname, "cyrus.expunge", NULL, 0);
+	r = dump_file(first, sync, pin, pout, oldname, "cyrus.expunge");
 	unlink(oldname);
 	if (r) return r;
     }
@@ -380,8 +379,7 @@ static int dump_annotations(const char *mailbox __attribute__((unused)),
 
 static int dump_file(int first, int sync,
 		     struct protstream *pin, struct protstream *pout,
-		     const char *filename, const char *ftag,
-		     const char *fbase, unsigned long flen)
+		     const char *filename, const char *ftag)
 {
     int filefd;
     const char *base;
@@ -391,43 +389,26 @@ static int dump_file(int first, int sync,
 
     /* map file */
     syslog(LOG_DEBUG, "wanting to dump %s", filename);
-    if (fbase) {
-	/* already mapped */
-	base = fbase;
-
-	/* we need to check real file size since actual mmap size may be larger */
-	if (stat(filename, &sbuf) == -1) {
-	    syslog(LOG_ERR, "IOERROR: stat on %s: %m", filename);
-	    fatal("can't stat message file", EC_OSFILE);
-	}
-	if ((unsigned long)sbuf.st_size > flen) {
-	   syslog(LOG_ERR, "IOERROR: size mismatch on %s", filename);
-	   return IMAP_SYS_ERROR;
-	}
-	len = sbuf.st_size;
+    filefd = open(filename, O_RDONLY, 0666);
+    if (filefd == -1) {
+	/* If an optional file doesn't exist, skip it */
+	if (errno == ENOENT) return 0;
+	syslog(LOG_ERR, "IOERROR: open on %s: %m", filename);
+	return IMAP_SYS_ERROR;
     }
-    else {
-	filefd = open(filename, O_RDONLY, 0666);
-	if (filefd == -1) {
-	    /* If an optional file doesn't exist, skip it */
-	    if (errno == ENOENT) return 0;
-	    syslog(LOG_ERR, "IOERROR: open on %s: %m", filename);
-	    return IMAP_SYS_ERROR;
-	}
     
-	if (fstat(filefd, &sbuf) == -1) {
-	    syslog(LOG_ERR, "IOERROR: fstat on %s: %m", filename);
-	    fatal("can't fstat message file", EC_OSFILE);
-	}	
-
-	base = NULL;
-	len = 0;
-
-	map_refresh(filefd, 1, &base, &len, sbuf.st_size, filename, NULL);
-
-	close(filefd);
+    if (fstat(filefd, &sbuf) == -1) {
+	syslog(LOG_ERR, "IOERROR: fstat on %s: %m", filename);
+	fatal("can't fstat message file", EC_OSFILE);
     }
 
+    base = NULL;
+    len = 0;
+
+    map_refresh(filefd, 1, &base, &len, sbuf.st_size, filename, NULL);
+
+    close(filefd);
+
     /* send: name, size, and contents */
     if (first) {
 	prot_printf(pout, " {" SIZE_T_FMT "}\r\n", strlen(ftag));
@@ -451,7 +432,7 @@ static int dump_file(int first, int sync,
 		    ftag, len, (sync ? "+" : ""));
     }
     prot_write(pout, base, len);
-    if (!fbase) map_free(&base, &len);
+    map_free(&base, &len);
 
     return 0;
 }
@@ -494,10 +475,12 @@ int dump_mailbox(const char *tag, struct mailbox *mailbox, uint32_t uid_start,
     if (!mbdir && errno == EACCES) {
 	syslog(LOG_ERR,
 	       "could not dump mailbox %s (permission denied)", mailbox->name);
+	mailbox_close(&mailbox);
 	return IMAP_PERMISSION_DENIED;
     } else if (!mbdir) {
 	syslog(LOG_ERR,
 	       "could not dump mailbox %s (unknown error)", mailbox->name);
+	mailbox_close(&mailbox);
 	return IMAP_SYS_ERROR;
     }
 
@@ -525,28 +508,7 @@ int dump_mailbox(const char *tag, struct mailbox *mailbox, uint32_t uid_start,
 
     /* Dump cyrus data files */
     for (df = data_files; df->metaname; df++) {
-	const char *fbase = NULL;
-	unsigned long flen = 0;
-
-	switch (df->metaname) {
-	case META_INDEX:
-	    if (mailbox->index_base) {
-		fbase = mailbox->index_base;
-		flen = mailbox->index_len;
-	    }
-	    break;
-
-	case META_CACHE:
-	    if (mailbox->cache_buf.s) {
-		fbase = mailbox->cache_buf.s;
-		flen = mailbox->cache_buf.len;
-	    }
-	    break;
-
-	default:
-	    break;
-	}
-
+	fname = mailbox_meta_fname(mailbox, df->metaname);
 	if (df->metaname == META_INDEX && oldversion < MAILBOX_MINOR_VERSION) {
 	    expunged_seq = seqset_init(mailbox->i.last_uid, SEQ_SPARSE);
 	    syslog(LOG_NOTICE, "%s downgrading index to version %d for XFER",
@@ -557,8 +519,7 @@ int dump_mailbox(const char *tag, struct mailbox *mailbox, uint32_t uid_start,
 	    if (r) goto done;
 
 	} else {
-	    fname = mailbox_meta_fname(mailbox, df->metaname);
-	    r = dump_file(first, !tag, pin, pout, fname, df->fname, fbase, flen);
+	    r = dump_file(first, !tag, pin, pout, fname, df->fname);
 	    if (r) goto done;
 	}
 
@@ -590,7 +551,7 @@ int dump_mailbox(const char *tag, struct mailbox *mailbox, uint32_t uid_start,
 	/* construct path/filename */
 	fname = mailbox_message_fname(mailbox, uid);
 
-	r = dump_file(0, !tag, pin, pout, fname, name, NULL, 0);
+	r = dump_file(0, !tag, pin, pout, fname, name);
 	if (r) goto done;
     }
 
@@ -633,7 +594,7 @@ int dump_mailbox(const char *tag, struct mailbox *mailbox, uint32_t uid_start,
 		fatal("unknown user data file", EC_OSFILE);
 	    }
 
-	    r = dump_file(0, !tag, pin, pout, fname, ftag, NULL, 0);
+	    r = dump_file(0, !tag, pin, pout, fname, ftag);
 	    free(fname);
 	    if (r) goto done;
 	}
@@ -675,7 +636,7 @@ int dump_mailbox(const char *tag, struct mailbox *mailbox, uint32_t uid_start,
 				 sieve_path, next->d_name);
 
 			/* dump file */
-			r = dump_file(0, !tag, pin, pout, filename, tag_fname, NULL, 0);
+			r = dump_file(0, !tag, pin, pout, filename, tag_fname);
 			if (r) goto done;
 		    }
 		}
@@ -687,10 +648,9 @@ int dump_mailbox(const char *tag, struct mailbox *mailbox, uint32_t uid_start,
 	    
     } /* end if user INBOX */
 
- done:
-
     prot_printf(pout,")\r\n");
 
+ done:
     prot_flush(pout);
 
     if (mbdir) closedir(mbdir);
@@ -708,14 +668,18 @@ static int cleanup_seen_cb(char *name,
     int r;
     struct seqset *seq = NULL;
     struct mailbox *mailbox = NULL;
-    struct seendata sd = SEENDATA_INITIALIZER;
+    struct seendata sd;
     unsigned recno;
     struct index_record record;
 
     r = mailbox_open_iwl(name, &mailbox);
     if (r) goto done;
 
-    /* read in the seen data from the seendb */
+    /* already got a RECENTUID, it's probably got seen data.  Either we
+     * had a race condition (unlikely) or we've just been unlucky */
+    if (mailbox->i.recentuid) goto done;
+
+    /* otherwise read in the seen data from the seendb */
     r = seen_read(seendb, mailbox->uniqueid, &sd);
     if (r) goto done;
 
@@ -726,8 +690,6 @@ static int cleanup_seen_cb(char *name,
     for (recno = 1; recno <= mailbox->i.num_records; recno++) {
 	if (mailbox_read_index_record(mailbox, recno, &record))
 	    continue;
-	if (record.system_flags & (FLAG_SEEN|FLAG_EXPUNGED))
-	    continue; /* no need to rewrite */
 	if (seqset_ismember(seq, record.uid)) {
 	    record.system_flags |= FLAG_SEEN;
 	    r = mailbox_rewrite_index_record(mailbox, &record);
@@ -737,14 +699,15 @@ static int cleanup_seen_cb(char *name,
 
     /* and the header values */
     mailbox_index_dirty(mailbox);
-    if (mailbox->i.recentuid < sd.lastuid)
-	mailbox->i.recentuid = sd.lastuid;
-    if (mailbox->i.recenttime < sd.lastread)
-	mailbox->i.recenttime = sd.lastread;
+    mailbox->i.recentuid = sd.lastuid;
+    mailbox->i.recenttime = sd.lastread;
+
+    /* commit everything */
+    mailbox_commit(mailbox);
 
  done:
-    seqset_free(seq);
-    mailbox_close(&mailbox);
+    if (seq) seqset_free(seq);
+    if (mailbox) mailbox_close(&mailbox);
     return r;
 }
 
@@ -755,19 +718,18 @@ static int cleanup_seen_subfolders(const char *mbname)
     char buf[MAX_MAILBOX_NAME];
     int r;
 
-    /* not owned by a user */
-    if (!userid) return 0;
-
     /* no need to do inbox, it will have upgraded OK, just
      * the subfolders */
 
-    snprintf(buf, sizeof(buf), "%s.*", mbname);
-
     r = seen_open(userid, SEEN_SILENT, &seendb);
-    if (!r) mboxlist_findall(NULL, buf, 1, NULL, NULL, cleanup_seen_cb, seendb);
-    seen_close(&seendb);
+    if (r) return 0; /* oh well, maybe they didn't have one */
 
-    return 0;
+    snprintf(buf, sizeof(buf), "%s.*", mbname);
+    r = mboxlist_findall(NULL, buf, 1, NULL, NULL, cleanup_seen_cb, seendb);
+
+    seen_close(seendb);
+
+    return r;
 }
 
 int undump_mailbox(const char *mbname, 
@@ -782,13 +744,8 @@ int undump_mailbox(const char *mbname,
     const char *sieve_path = NULL;
     int sieve_usehomedir = config_getswitch(IMAPOPT_SIEVEUSEHOMEDIR);
     const char *userid = NULL;
-    char *annotation = NULL;
-    char *contenttype = NULL;
-    char *content = NULL;
-    char *seen_file = NULL;
-    char *mboxkey_file = NULL;
-    uquota_t old_quota_used = 0;
-    int quotalimit = -1;
+    char *annotation, *contenttype, *content;
+    char *seen_file = NULL, *mboxkey_file = NULL;
 
     memset(&file, 0, sizeof(file));
     memset(&data, 0, sizeof(data));
@@ -811,12 +768,12 @@ int undump_mailbox(const char *mbname,
     
     /* We should now have a number or a NIL */
     c = getword(pin, &data);
-    if (!strcmp(data.s, "NIL")) {
+    if(!strcmp(data.s, "NIL")) {
 	/* Remove any existing quotaroot */
 	mboxlist_unsetquota(mbname);
-    } else if (sscanf(data.s, "%d", &quotalimit) == 1) {
-	/* Set a Quota (may be -1 for "unlimited") */ 
-	mboxlist_setquota(mbname, quotalimit, 0);
+    } else if(imparse_isnumber(data.s)) {
+	/* Set a Quota */ 
+	mboxlist_setquota(mbname, atoi(data.s), 0);
     } else {
 	/* Huh? */
 	buf_free(&data);
@@ -841,9 +798,6 @@ int undump_mailbox(const char *mbname,
     }
     if(r) goto done;
 
-    /* track quota use */
-    old_quota_used = mailbox->i.quota_mailbox_used;
-
     while(1) {
 	char fnamebuf[MAX_MAILBOX_PATH + 1024];
 	int isnowait, sawdigit;
@@ -932,9 +886,6 @@ int undump_mailbox(const char *mbname,
 	    free(annotation);
 	    free(content);
 	    free(contenttype);
-	    annotation = NULL;
-	    content = NULL;
-	    contenttype = NULL;
 
 	    c = prot_getc(pin);
 	    if(c == ')') break; /* that was the last item */
@@ -1101,7 +1052,7 @@ int undump_mailbox(const char *mbname,
 	    struct seen *seendb = NULL;
 	    r = seen_open(userid, SEEN_CREATE, &seendb);
 	    if (!r) r = seen_merge(seendb, fnamebuf);
-	    seen_close(&seendb);
+	    if (seendb) seen_close(seendb);
 
 	    free(seen_file);
 	    seen_file = NULL;
@@ -1141,7 +1092,7 @@ int undump_mailbox(const char *mbname,
     if (curfile >= 0) close(curfile);
     /* we fiddled the files under the hood, so we can't do anything
      * BUT close it */
-    mailbox_close(&mailbox);
+    if (mailbox) mailbox_close(&mailbox);
 
     /* let's make sure the modification times are right */
     if (!r) {
@@ -1157,28 +1108,6 @@ int undump_mailbox(const char *mbname,
 	    goto done2;
 	}
 
-	/* update the quota if necessary */
-	if (mailbox->quotaroot &&
-	    old_quota_used != mailbox->i.quota_mailbox_used) {
-	    struct txn *tid = NULL;
-	    struct quota q;
-
-	    q.root = mailbox->quotaroot;
-	    r = quota_read(&q, &tid, 1);
-	    if (!r) {
-		q.used += mailbox->i.quota_mailbox_used - old_quota_used;
-		r = quota_write(&q, &tid);
-	    }
-	    if (!r) quota_commit(&tid);
-	    else {
-		quota_abort(&tid);
-		syslog(LOG_ERR, "LOSTQUOTA: unable to record add of " 
-		       UQUOTA_T_FMT " bytes in quota %s",
-		       mailbox->i.quota_mailbox_used - old_quota_used,
-		       mailbox->quotaroot);
-	    }
-	}
-
 	for (recno = 1; recno <= mailbox->i.num_records; recno++) {
 	    r = mailbox_read_index_record(mailbox, recno, &record);
 	    if (r) continue;
@@ -1196,7 +1125,7 @@ int undump_mailbox(const char *mbname,
 
  done2:
     /* just in case we failed during the modifications, close again */
-    mailbox_close(&mailbox);
+    if (mailbox) mailbox_close(&mailbox);
     
     free(annotation);
     free(content);
