@@ -73,7 +73,6 @@
 #include "xmalloc.h"
 #include "xstrlcpy.h"
 #include "xstrlcat.h"
-#include "strarray.h"
 #include "global.h"
 #include "retry.h"
 
@@ -85,6 +84,86 @@ struct msg {
     int encode;
 };
 
+/* cyrus.cache file item buffer */
+struct ibuf {
+    char *start, *end, *last;
+};
+    
+/*
+ * Parsed form of a body-part
+ */
+struct body {
+    /* Content-* header information */
+    char *type;
+    char *subtype;
+    struct param *params;
+    char *id;
+    char *description;
+    char *encoding;
+    char *md5;
+    char *disposition;
+    struct param *disposition_params;
+    struct param *language;
+    char *location;
+
+    /* Location/size information */
+    long header_offset;
+    long header_size;
+    long header_lines;
+    long content_offset;
+    long content_size;
+    long content_lines;
+    long boundary_size;		/* Size of terminating boundary */
+    long boundary_lines;
+
+    int numparts;		/* For multipart types */
+    struct body *subpart;	/* For message/rfc822 and multipart types */
+
+    /*
+     * Other header information.
+     * Only meaningful for body-parts at top level or
+     * enclosed in message/rfc-822
+     */
+    char *date;
+    char *subject;
+    struct address *from;
+    struct address *sender;
+    struct address *reply_to;
+    struct address *to;
+    struct address *cc;
+    struct address *bcc;
+    char *in_reply_to;
+    char *message_id;
+    char *received_date;
+
+    /*
+     * Cached headers.  Only filled in at top-level
+     */
+    struct ibuf cacheheaders;
+
+    /*
+     * decoded body.  Filled in as needed.
+     */
+    char *decoded_body;
+
+    /* Message GUID. Only filled in at top level */
+    struct message_guid guid;
+};
+
+/* List of Content-type parameters */
+struct param {
+    struct param *next;
+    char *attribute;
+    char *value;
+};
+
+/* List of pending multipart boundaries */
+struct boundary {
+    char **id;
+    int count;
+    int alloc;
+};
+
 /* (draft standard) MIME tspecials */
 #define TSPECIALS "()<>@,;:\\\"/[]?="
 
@@ -94,12 +173,12 @@ struct msg {
 static int message_parse_body P((struct msg *msg,
 				 struct body *body,
 				 char *defaultContentType,
-				 strarray_t *boundaries));
+				 struct boundary *boundaries));
 
 static int message_parse_headers P((struct msg *msg,
 				    struct body *body,
 				    char *defaultContentType,
-				    strarray_t *boundaries));
+				    struct boundary *boundaries));
 static void message_parse_address P((char *hdr, struct address **addrp));
 static void message_parse_encoding P((char *hdr, char **hdrp));
 static void message_parse_charset P((struct body *body, int *encoding, int *charset));
@@ -115,14 +194,14 @@ static void message_parse_received_date P((char *hdr, char **hdrp));
 
 static void message_parse_multipart P((struct msg *msg,
 				       struct body *body,
-				       strarray_t *boundaries));
+				       struct boundary *boundaries));
 static void message_parse_content P((struct msg *msg,
 				     struct body *body,
-				     strarray_t *boundaries));
+				     struct boundary *boundaries));
 
 static char *message_getline P((char *s, unsigned n, struct msg *msg));
-static int message_pendingboundary P((const char *s, int len,
-				      strarray_t *boundaries));
+static int message_pendingboundary P((const char *s, char **boundaries,
+				      int *boundaryct));
 
 static void message_write_envelope P((struct ibuf *ibuf, struct body *body));
 static void message_write_body P((struct ibuf *ibuf, struct body *body,
@@ -268,15 +347,13 @@ int message_parse(const char *fname, struct index_record *record)
     if (!f) return IMAP_IOERROR;
 
     r = message_parse_file(f, NULL, NULL, &body);
-    if (!r) r = message_create_record(record, body);
+    if (!r) {
+        r = message_create_record(record, body);
+        message_free_body(body);
+    }
 
     if (f) fclose(f);
 
-    if (body) {
-	message_free_body(body);
-	free(body);
-    }
-
     return r;
 }
 
@@ -363,9 +440,7 @@ int message_parse_binary_file(FILE *infile, struct body **body)
 
     if (!*body) *body = (struct body *) xmalloc(sizeof(struct body));
     message_parse_body(&msg, *body,
-		       DEFAULT_CONTENT_TYPE, (strarray_t *)0);
-
-    message_guid_generate(&(*body)->guid, msg.base, msg.len);
+		       DEFAULT_CONTENT_TYPE, (struct boundary *)0);
 
     lseek(fd, 0L, SEEK_SET);
     n = retry_write(fd, msg.base, msg.len);
@@ -395,7 +470,7 @@ int message_parse_mapped(const char *msg_base, unsigned long msg_len,
     msg.encode = 0;
 
     message_parse_body(&msg, body,
-		       DEFAULT_CONTENT_TYPE, (strarray_t *)0);
+		       DEFAULT_CONTENT_TYPE, (struct boundary *)0);
 
     message_guid_generate(&body->guid, msg_base, msg_len);
 
@@ -489,23 +564,35 @@ message_create_record(record, body)
 struct index_record *record;
 struct body *body;
 {
-    if (!record->internaldate) {
-	if (config_getenum(IMAPOPT_INTERNALDATE_HEURISTIC) 
-	    == IMAP_ENUM_INTERNALDATE_HEURISTIC_RECEIVEDHEADER &&
-	    body->received_date)
-	    record->internaldate = message_parse_date(
-			body->received_date,
-			PARSE_DATE|PARSE_TIME|PARSE_ZONE|
-			PARSE_NOCREATE|PARSE_GMT);
+    if (config_getenum(IMAPOPT_INTERNALDATE_HEURISTIC) 
+	    == IMAP_ENUM_INTERNALDATE_HEURISTIC_RECEIVEDHEADER) {
+	time_t newdate = 0;
+	if (body->received_date)
+	    newdate = message_parse_date(body->received_date,
+		PARSE_DATE|PARSE_TIME|PARSE_ZONE|PARSE_NOCREATE|PARSE_GMT);
+	if (newdate)
+	    record->internaldate = newdate;
     }
 
+    if (!record->internaldate)
+	record->internaldate = time(NULL);
 
+    /* used for sent time searching, truncated to day with no TZ */
     record->sentdate = message_parse_date(body->date, PARSE_NOCREATE);
+    if (!record->sentdate) {
+	struct tm *tm = localtime(&record->internaldate);
+	/* truncate to the day */
+	tm->tm_sec = 0;
+	tm->tm_min = 0;
+	tm->tm_hour = 0;
+	record->sentdate = mktime(tm);
+    }
 
     /* used for sent time sorting, full gmtime of Date: header */
-    record->gmtime = message_parse_date(body->date,
-		     PARSE_DATE|PARSE_TIME|PARSE_ZONE|
-		     PARSE_NOCREATE|PARSE_GMT);
+    record->gmtime =
+	message_parse_date(body->date, PARSE_DATE|PARSE_TIME|PARSE_ZONE|PARSE_NOCREATE|PARSE_GMT);
+    if (!record->gmtime)
+	record->gmtime = record->internaldate;
 
     record->size = body->header_size + body->content_size;
     record->header_size = body->header_size;
@@ -525,16 +612,18 @@ message_parse_body(msg, body, defaultContentType, boundaries)
 struct msg *msg;
 struct body *body;
 char *defaultContentType;
-strarray_t *boundaries;
+struct boundary *boundaries;
 {
-    strarray_t newboundaries = STRARRAY_INITIALIZER;
+    struct boundary newboundaries;
     int sawboundary;
 
     memset(body, 0, sizeof(struct body));
+    newboundaries.id = 0;
 
     /* No passed-in boundary structure, create a new, empty one */
     if (!boundaries) {
 	boundaries = &newboundaries;
+	boundaries->alloc = boundaries->count = 0;
 	/* We're at top-level--set up to store cached headers */
 	message_ibuf_init(&body->cacheheaders);
     }
@@ -578,7 +667,7 @@ strarray_t *boundaries;
     }
 
     /* Free up boundary storage if necessary */
-    strarray_fini(&newboundaries);
+    if (newboundaries.id) free(newboundaries.id);
 
     return 0;
 }
@@ -593,7 +682,7 @@ message_parse_headers(msg, body, defaultContentType,
 struct msg *msg;
 struct body *body;
 char *defaultContentType;
-strarray_t *boundaries;
+struct boundary *boundaries;
 {
     static int alloced = 0;
     static char *headers;
@@ -619,11 +708,9 @@ strarray_t *boundaries;
 	   (next[-1] != '\n' ||
 	    (*next != '\r' || next[1] != '\n'))) {
 
-	len = strlen(next);
-
 	if (next[-1] == '\n' && *next == '-' &&
-	    message_pendingboundary(next, len, boundaries)) {
-	    body->boundary_size = len;
+	    message_pendingboundary(next, boundaries->id, &boundaries->count)) {
+	    body->boundary_size = strlen(next);
 	    body->boundary_lines++;
 	    if (next - 1 > headers) {
 		body->boundary_size += 2;
@@ -637,6 +724,7 @@ strarray_t *boundaries;
 	    break;
 	}
 
+	len = strlen(next);
 	left -= len;
 	next += len;
 
@@ -1732,13 +1820,12 @@ static void
 message_parse_multipart(msg, body, boundaries)
 struct msg *msg;
 struct body *body;
-strarray_t *boundaries;
+struct boundary *boundaries;
 {
     struct body preamble, epilogue;
     struct param *boundary;
     char *defaultContentType = DEFAULT_CONTENT_TYPE;
     int i, depth;
-    int limit = config_getint(IMAPOPT_BOUNDARY_LIMIT);
 
     memset(&preamble, 0, sizeof(struct body));
     memset(&epilogue, 0, sizeof(struct body));
@@ -1758,16 +1845,22 @@ strarray_t *boundaries;
 	return;
     }
 
+    /* Expand boundaries array if necessary */
+    if (boundaries->count == boundaries->alloc) {
+	boundaries->alloc += 20;
+	boundaries->id = (char **)xrealloc((char *)boundaries->id,
+					   boundaries->alloc * sizeof(char *));
+    }
+    
     /* Add the new boundary id */
-    strarray_append(boundaries, boundary->value);
+    boundaries->id[boundaries->count++] = boundary->value;
     depth = boundaries->count;
 
     /* Parse preamble */
     message_parse_content(msg, &preamble, boundaries);
 
     /* Parse the component body-parts */
-    while (boundaries->count == depth &&
-	    (limit == 0 ? 1 : boundaries->count < limit)) {
+    while (boundaries->count == depth) {
 	body->subpart = (struct body *)xrealloc((char *)body->subpart,
 				 (body->numparts+1)*sizeof(struct body));
 	message_parse_body(msg, &body->subpart[body->numparts++],
@@ -1828,11 +1921,6 @@ strarray_t *boundaries;
      */
     body->boundary_size += epilogue.boundary_size;
     body->boundary_lines += epilogue.boundary_lines;
-
-    /* check if we've hit a limit and flag it */
-    if (limit && depth == limit) {
-	syslog(LOG_ERR, "ERROR: mime boundary limit %i exceeded, not parsing anymore", limit);
-    }
 }
 
 /*
@@ -1842,7 +1930,7 @@ static void
 message_parse_content(msg, body, boundaries)
 struct msg *msg;
 struct body *body;
-strarray_t *boundaries;
+struct boundary *boundaries;
 {
     const char *line, *endline;
     unsigned long s_offset = msg->offset;
@@ -1866,7 +1954,7 @@ strarray_t *boundaries;
 	msg->offset += len;
 
 	if (line[0] == '-' && line[1] == '-' &&
-	    message_pendingboundary(line, len, boundaries)) {
+	    message_pendingboundary(line, boundaries->id, &boundaries->count)) {
 	    body->boundary_size = len;
 	    body->boundary_lines++;
 	    if (body->content_lines) {
@@ -1989,33 +2077,22 @@ struct msg *msg;
  * If we hit a terminating boundary, the integer pointed to by
  * 'boundaryct' is modified appropriately.
  */
-static int message_pendingboundary(s, slen, boundaries)
+static int message_pendingboundary(s, boundaries, boundaryct)
 const char *s;
-int slen;
-strarray_t *boundaries;
+char **boundaries;
+int *boundaryct;
 {
     int i, len;
     int rfc2046_strict = config_getswitch(IMAPOPT_RFC2046_STRICT);
-    const char *bbase;
-    int blen;
 
-    /* skip initial '--' */
-    if (slen < 2) return 0;
-    if (s[0] != '-' || s[1] != '-') return 0;
-    bbase = s + 2;
-    blen = slen - 2;
+    if (s[0] != '-' || s[1] != '-') return(0);
+    s+=2;
 
-    for (i = 0; i < boundaries->count ; ++i) {
-	len = strlen(boundaries->data[i]);
-	/* basic sanity check and overflow protection */
-	if (blen < len) continue;
-
-	if (!strncmp(bbase, boundaries->data[i], len)) {
-	    /* trailing '--', it's the end of this part */
-	    if (blen >= len+2 && bbase[len] == '-' && bbase[len+1] == '-')
-		strarray_truncate(boundaries, i);
-	    else if (!rfc2046_strict && blen > len+1 &&
-		     bbase[len] && !Uisspace(bbase[len])) {
+    for (i=0; i < *boundaryct; ++i) {
+	len = strlen(boundaries[i]);
+        if (!strncmp(s, boundaries[i], len)) {
+            if (s[len] == '-' && s[len+1] == '-') *boundaryct = i;
+	    else if (!rfc2046_strict && s[len] && !Uisspace(s[len])) {
 		/* Allow substring matches in the boundary.
 		 *
 		 * If rfc2046_strict is enabled, boundaries containing
@@ -2025,10 +2102,10 @@ strarray_t *boundaries;
 		 */
 		continue;
 	    }
-	    return 1;
-	}
+            return(1);
+        }
     }
-    return 0;
+    return(0);
 }
 
 /*
