@@ -496,8 +496,8 @@ static int mupdate_proxy_policy(sasl_conn_t *conn,
 }
 
 static struct sasl_callback mysasl_cb[] = {
-    { SASL_CB_GETOPT, (mysasl_cb_ft *) &mysasl_config, NULL },
-    { SASL_CB_PROXY_POLICY, (mysasl_cb_ft *) &mupdate_proxy_policy, NULL },
+    { SASL_CB_GETOPT, &mysasl_config, NULL },
+    { SASL_CB_PROXY_POLICY, &mupdate_proxy_policy, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
@@ -922,9 +922,6 @@ mupdate_docmd_result_t docmd(struct conn *c)
 	if (!strcmp(c->cmd.s, "Starttls")) {
 	    CHECKNEWLINE(c, ch);
 	    
-	    /* XXX  discard any input pipelined after STARTTLS */
-	    prot_flush(c->pin);
-
 	    if (!tls_enabled()) {
 		/* we don't support starttls */
 		goto badcmd;
@@ -1525,7 +1522,6 @@ void cmd_authenticate(struct conn *C,
 {
     int r, sasl_result;
     const void *val;
-    int failedloginpause;
 
     r = saslserver(C->saslconn, mech, clientstart, "", "", "",
 		   C->pin, C->pout, &sasl_result, NULL);
@@ -1546,10 +1542,7 @@ void cmd_authenticate(struct conn *C,
 			tag, errorstring ? errorstring : "");
 	    break;
 	default:
-	    failedloginpause = config_getint(IMAPOPT_FAILEDLOGINPAUSE);
-	    if (failedloginpause != 0) {
-	        sleep(failedloginpause);
-	    }
+	    sleep(3);
 	
 	    syslog(LOG_ERR, "badlogin: %s %s %s",
 		   C->clienthost,
