@@ -313,9 +313,9 @@ static int popd_proxy_policy(sasl_conn_t *conn,
 }
 
 static struct sasl_callback mysasl_cb[] = {
-    { SASL_CB_GETOPT, (mysasl_cb_ft *) &mysasl_config, NULL },
-    { SASL_CB_PROXY_POLICY, (mysasl_cb_ft *) &popd_proxy_policy, (void*) &popd_proxyctx },
-    { SASL_CB_CANON_USER, (mysasl_cb_ft *) &popd_canon_user, NULL },
+    { SASL_CB_GETOPT, &mysasl_config, NULL },
+    { SASL_CB_PROXY_POLICY, &popd_proxy_policy, (void*) &popd_proxyctx },
+    { SASL_CB_CANON_USER, &popd_canon_user, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
@@ -816,7 +816,6 @@ static int expunge_deleted(void)
     struct index_record record;
     uint32_t msgno;
     int r = 0;
-    int numexpunged = 0;
 
     /* loop over all known messages looking for deletes */
     for (msgno = 1; msgno <= popd_exists; msgno++) {
@@ -834,7 +833,6 @@ static int expunge_deleted(void)
 
 	/* mark expunged */
 	record.system_flags |= FLAG_EXPUNGED;
-        numexpunged++;
 
 	/* store back to the mailbox */
 	r = mailbox_rewrite_index_record(popd_mailbox, &record);
@@ -846,11 +844,6 @@ static int expunge_deleted(void)
 	       popd_mailbox->name, msgno, popd_msg[msgno].uid);
     }
 
-    if (!r && (numexpunged > 0)) {
-	syslog(LOG_NOTICE, "Expunged %d messages from %s",
-	       numexpunged, popd_mailbox->name);
-    }
-
     return r;
 }
 
@@ -965,13 +958,17 @@ static void cmdloop(void)
 		    popd_mailbox->i.pop3_last_login = popd_login_time;
 		}
 
-		/* look for deleted messages */
+		/* check for changes */
+		for (msgno = 1; msgno <= popd_exists; msgno++)
+		    if (popd_msg[msgno].seen) break;
+
+		if (msgno <= popd_exists)
+		    update_seen();
+
+		/* process looking for deleted messages */
 		expunge_deleted();
 
-		/* update seen data */
-		update_seen();
-
-		/* unlock will commit changes */
+		mailbox_commit(popd_mailbox);
 		mailbox_unlock_index(popd_mailbox, NULL);
 
 done:
@@ -1013,9 +1010,6 @@ done:
 		if (arg) {
 		    prot_printf(popd_out, "-ERR Unexpected extra argument\r\n");
 		} else {
-		    /* XXX  discard any input pipelined after STLS */
-		    prot_flush(popd_in);
-
 		    cmd_starttls(0);
 		}
 	    }
@@ -1103,8 +1097,6 @@ done:
 	    uint32_t lines;
 	    int r;
 
-	    if (!p)
-		p = "";
 	    while (*p && Uisspace(*p)) {
 		p++;
 	    }
@@ -1298,7 +1290,6 @@ static void cmd_apop(char *response)
 {
     int sasl_result;
     const void *canon_user;
-    int failedloginpause;
 
     assert(response != NULL);
 
@@ -1320,10 +1311,7 @@ static void cmd_apop(char *response)
 	       popd_clienthost, popd_apop_chal,
 	       sasl_errdetail(popd_saslconn));
 	
-	failedloginpause = config_getint(IMAPOPT_FAILEDLOGINPAUSE);
-	if (failedloginpause != 0) {
-	    sleep(failedloginpause);
-	}
+	sleep(3);      
 
 	/* Don't allow user probing */
 	if (sasl_result == SASL_NOUSER) sasl_result = SASL_BADAUTH;
@@ -1357,9 +1345,9 @@ static void cmd_apop(char *response)
     }
     popd_userid = xstrdup((const char *) canon_user);
     
-    syslog(LOG_NOTICE, "login: %s %s%s APOP%s %s SESSIONID=<%s>", popd_clienthost,
+    syslog(LOG_NOTICE, "login: %s %s%s APOP%s %s", popd_clienthost,
 	   popd_userid, popd_subfolder ? popd_subfolder : "",
-	   popd_starttls_done ? "+TLS" : "", "User logged in", session_id());
+	   popd_starttls_done ? "+TLS" : "", "User logged in");
 
     popd_authstate = auth_newstate(popd_userid);
 
@@ -1405,8 +1393,6 @@ void cmd_user(char *user)
 
 void cmd_pass(char *pass)
 {
-    int failedloginpause;
-
     if (!popd_userid) {
 	prot_printf(popd_out, "-ERR [AUTH] Must give USER command\r\n");
 	return;
@@ -1426,9 +1412,9 @@ void cmd_pass(char *pass)
 	    return;
 	}
 
-	syslog(LOG_NOTICE, "login: %s %s%s KPOP%s %s SESSIONID=<%s>", popd_clienthost,
+	syslog(LOG_NOTICE, "login: %s %s%s KPOP%s %s", popd_clienthost,
 	       popd_userid, popd_subfolder ? popd_subfolder : "",
-	       popd_starttls_done ? "+TLS" : "", "User logged in", session_id());
+	       popd_starttls_done ? "+TLS" : "", "User logged in");
 
 	openinbox();
 	return;
@@ -1456,10 +1442,7 @@ void cmd_pass(char *pass)
 			    strlen(pass))!=SASL_OK) { 
 	syslog(LOG_NOTICE, "badlogin: %s plaintext %s %s",
 	       popd_clienthost, popd_userid, sasl_errdetail(popd_saslconn));
-	failedloginpause = config_getint(IMAPOPT_FAILEDLOGINPAUSE);
-	if (failedloginpause != 0) {
-	    sleep(failedloginpause);
-	}
+	sleep(3);
 	prot_printf(popd_out, "-ERR [AUTH] Invalid login\r\n");
 	free(popd_userid);
 	popd_userid = 0;
@@ -1503,11 +1486,6 @@ void cmd_pass(char *pass)
 	}
     }
 
-    /* popd_authstate may have been set as a side effect
-     * of sasl_checkpass() calling mysasl_proxy_policy */
-    if (popd_authstate)
-	auth_freestate(popd_authstate);
-
     popd_authstate = auth_newstate(popd_userid);
 
     openinbox();
@@ -1573,7 +1551,6 @@ void cmd_auth(char *arg)
     char *authtype;
     const void *val;
     const char *canon_user;
-    int failedloginpause;
 
     /* if client didn't specify an argument we give them the list
      *
@@ -1647,10 +1624,7 @@ void cmd_auth(char *arg)
 		       popd_clienthost, authtype);
 	    }
 
-	    failedloginpause = config_getint(IMAPOPT_FAILEDLOGINPAUSE);
-	    if (failedloginpause != 0) {
-	        sleep(failedloginpause);
-	    }
+	    sleep(3);
 
 	    /* Don't allow user probing */
 	    if (sasl_result == SASL_NOUSER) sasl_result = SASL_BADAUTH;
@@ -1701,9 +1675,9 @@ void cmd_auth(char *arg)
     } else {
 	popd_userid = xstrdup(canon_user);
     }
-    syslog(LOG_NOTICE, "login: %s %s%s %s%s %s SESSIONID=<%s>", popd_clienthost,
+    syslog(LOG_NOTICE, "login: %s %s%s %s%s %s", popd_clienthost,
 	   popd_userid, popd_subfolder ? popd_subfolder : "",
-	   authtype, popd_starttls_done ? "+TLS" : "", "User logged in", session_id());
+	   authtype, popd_starttls_done ? "+TLS" : "", "User logged in");
 
     sasl_getprop(popd_saslconn, SASL_SSF, &val);
     saslprops.ssf = *((sasl_ssf_t *) val);
@@ -1753,10 +1727,6 @@ int openinbox(void)
 	    IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
 	log_level = LOG_INFO;
     }
-    else if (!r && (mbentry.mbtype & MBTYPE_DELETED)) {
-	r = IMAP_MAILBOX_NONEXISTENT;
-	log_level = LOG_INFO;
-    }
     if (r) {
 	sleep(3);
 	syslog(log_level, "Unable to locate maildrop %s: %s",
@@ -1767,19 +1737,6 @@ int openinbox(void)
 	goto fail;
     }
 
-    if (mbentry.mbtype & MBTYPE_RESERVE) r = IMAP_MAILBOX_RESERVED;
-    if (mbentry.mbtype & MBTYPE_MOVING) r = IMAP_MAILBOX_MOVED;
-    if (r) {
-	sleep(3);
-	log_level = LOG_INFO;
-	syslog(log_level, "Unable to open maildrop %s: %s",
-	       inboxname, error_message(r));
-	prot_printf(popd_out,
-		    "-ERR [SYS/TEMP] Unable to open maildrop: %s\r\n",
-		    error_message(r));
-	goto fail;
-    }
-
     if (mbentry.mbtype & MBTYPE_REMOTE) {
 	/* remote mailbox */
 	char *server = mbentry.partition;
@@ -1829,6 +1786,7 @@ int openinbox(void)
 	popd_login_time = time(0);
 
 	r = mailbox_open_iwl(inboxname, &popd_mailbox);
+	popd_myrights = cyrus_acl_myrights(popd_authstate, popd_mailbox->acl);
 	if (r) {
 	    sleep(3);
 	    syslog(log_level, "Unable to open maildrop %s: %s",
@@ -1838,7 +1796,6 @@ int openinbox(void)
 			error_message(r));
 	    goto fail;
 	}
-	popd_myrights = cyrus_acl_myrights(popd_authstate, popd_mailbox->acl);
 	if (config_popuseacl && !(popd_myrights & ACL_READ)) {
 	    r = (popd_myrights & ACL_LOOKUP) ?
 		 IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
@@ -2077,7 +2034,7 @@ static int update_seen(void)
 
     /* we know this mailbox must be owned by the user, because 
      * all POP mailboxes are */
-    for (i = 1; i <= popd_exists; i++) {
+    for (i = 0; i < popd_exists; i++) {
 	if (!popd_msg[i].seen)
 	    continue; /* don't even need to check */
 	if (mailbox_read_index_record(popd_mailbox, popd_msg[i].recno, &record))
