@@ -94,7 +94,7 @@
 #include "cyrusdb.h"
 
 /* signal to config.c */
-const int config_need_data = CONFIG_NEED_PARTITION_DATA;
+const int config_need_data = 0;  /* YYY */
 
 /* ====================================================================== */
 
@@ -113,7 +113,7 @@ static struct namespace   sync_namespace;
 static int verbose         = 0;
 static int verbose_logging = 0;
 static int connect_once    = 0;
-static int background      = 0;
+static int foreground      = 0;
 static int do_compress     = 0;
 
 static struct protocol_t csync_protocol =
@@ -273,22 +273,20 @@ static int mark_missing (struct dlist *kin,
     if (!kl) return 0;
 
     if (strcmp(kl->name, "MISSING")) {
-	syslog(LOG_ERR, "SYNCERROR: Illegal response to RESERVE: %s",
-	       kl->name);
+	syslog(LOG_ERR, "Illegal response to RESERVE: %s", kl->name);
 	return IMAP_PROTOCOL_BAD_PARAMETERS;
     }
 
     /* unmark each missing item */
     for (ki = kl->head; ki; ki = ki->next) {
 	if (!message_guid_decode(&tmp_guid, ki->sval)) {
-	    syslog(LOG_ERR, "SYNCERROR: reserve: failed to parse GUID %s",
-		   ki->sval);
+	    syslog(LOG_ERR, "RESERVE: failed to parse GUID %s", ki->sval);
 	    return IMAP_PROTOCOL_BAD_PARAMETERS;
         }
 
 	msgid = sync_msgid_lookup(part_list, &tmp_guid);
 	if (!msgid) {
-	    syslog(LOG_ERR, "SYNCERROR: reserve: Got unexpected GUID %s", ki->sval);
+	    syslog(LOG_ERR, "RESERVE: Got unexpected GUID %s", ki->sval);
 	    return IMAP_PROTOCOL_BAD_PARAMETERS;
 	}
 
@@ -473,8 +471,7 @@ static int response_parse(const char *cmd,
 
  parse_err:
     dlist_free(&kin);
-    syslog(LOG_ERR, "SYNCERROR: %s: Invalid response %s",
-	   cmd, dlist_lastkey());
+    syslog(LOG_ERR, "%s: Invalid response %s", cmd, dlist_lastkey());
     return IMAP_PROTOCOL_BAD_PARAMETERS;
 }
 
@@ -700,37 +697,33 @@ static int user_sub(const char *userid, const char *mboxname)
 static int copy_local(struct mailbox *mailbox, unsigned long uid)
 {
     uint32_t recno;
-    struct index_record oldrecord;
+    struct index_record record;
+    char *oldfname, *newfname;
     int r;
 
     for (recno = 1; recno <= mailbox->i.num_records; recno++) {
-	r = mailbox_read_index_record(mailbox, recno, &oldrecord);
+	r = mailbox_read_index_record(mailbox, recno, &record);
 	if (r) return r;
+	if (record.uid == uid) {
+	    /* store the old record, expunged */
+	    record.system_flags |= FLAG_EXPUNGED;
+	    r = mailbox_rewrite_index_record(mailbox, &record);
+	    if (r) return r;
 
-	/* found the record, renumber it */
-	if (oldrecord.uid == uid) {
-	    char *oldfname, *newfname;
-	    struct index_record newrecord;
-
-	    /* create the new record as a clone of the old record */
-	    newrecord = oldrecord;
-	    newrecord.uid = mailbox->i.last_uid + 1;
+	    /* create the new record */
+	    record.system_flags &= ~FLAG_EXPUNGED;
+	    record.uid = mailbox->i.last_uid + 1;
 
 	    /* copy the file in to place */
-	    oldfname = xstrdup(mailbox_message_fname(mailbox, oldrecord.uid));
-	    newfname = xstrdup(mailbox_message_fname(mailbox, newrecord.uid));
+	    oldfname = xstrdup(mailbox_message_fname(mailbox, uid));
+	    newfname = xstrdup(mailbox_message_fname(mailbox, record.uid));
 	    r = mailbox_copyfile(oldfname, newfname, 0);
 	    free(oldfname);
 	    free(newfname);
 	    if (r) return r;
 
-	    /* append the new record */
-	    r = mailbox_append_index_record(mailbox, &newrecord);
-	    if (r) return r;
-
-	    /* and expunge the old record */
-	    oldrecord.system_flags |= FLAG_EXPUNGED;
-	    r = mailbox_rewrite_index_record(mailbox, &oldrecord);
+	    /* and append the new record (a clone apart from the EXPUNGED flag) */
+	    r = mailbox_append_index_record(mailbox, &record);
 
 	    /* done - return */
 	    return r;
@@ -747,13 +740,7 @@ static int fetch_file(struct mailbox *mailbox, unsigned long uid,
     const char *cmd = "FETCH";
     struct dlist *kin = NULL;
     struct dlist *kl;
-    int r = 0;
-    const char *fname = dlist_reserve_path(mailbox->part, &rp->guid);
-    struct stat sbuf;
-
-    /* already reserved? great */
-    if (stat(fname, &sbuf) == 0)
-	return 0;
+    int r;
 
     kl = dlist_new(cmd);
     dlist_atom(kl, "MBOXNAME", mailbox->name);
@@ -772,7 +759,7 @@ static int fetch_file(struct mailbox *mailbox, unsigned long uid,
 	goto done;
     }
 
-    if (!message_guid_equal(&kl->gval, &rp->guid))
+    if (!message_guid_compare(&kl->gval, &rp->guid))
 	r = IMAP_MAILBOX_CRC;
 
 done:
@@ -791,10 +778,12 @@ static int copy_remote(struct mailbox *mailbox, unsigned long uid,
 	r = parse_upload(ki, mailbox, &record);
 	if (r) return r;
 	if (record.uid == uid) {
-	    /* choose the destination UID */
+	    /* find the destination UID */
 	    record.uid = mailbox->i.last_uid + 1;
 
-	    /* already fetched the file in the parse phase */
+	    /* upload the file */
+	    r = fetch_file(mailbox, uid, &record);
+	    if (r) return r;
 
 	    /* append the file */
 	    r = sync_append_copyfile(mailbox, &record);
@@ -823,16 +812,10 @@ static int copyback_one_record(struct mailbox *mailbox,
 	 * (has been cleaned out locally) or an error.
 	 * In the error case we copy back, stale
 	 * we remove from the replica */
-	if (rp->modseq < mailbox->i.deletedmodseq) {
-	    if (kaction)
-		dlist_num(kaction, "EXPUNGE", rp->uid);
-	}
-	else {
-	    r = fetch_file(mailbox, rp->uid, rp);
-	    if (r) return r;
-	    if (kaction)
-		dlist_num(kaction, "COPYBACK", rp->uid);
-	}
+	if (rp->modseq < mailbox->i.deletedmodseq)
+	    dlist_num(kaction, "EXPUNGE", rp->uid);
+	else
+	    dlist_num(kaction, "COPYBACK", rp->uid);
     }
 
     /* otherwise we can pull it in with the same UID,
@@ -842,8 +825,6 @@ static int copyback_one_record(struct mailbox *mailbox,
 	/* grab the file */
 	r = fetch_file(mailbox, rp->uid, rp);
 	if (r) return r;
-	/* make sure we're actually making changes now */
-	if (!kaction) return 0;
 	/* append the file */
 	r = sync_append_copyfile(mailbox, rp);
 	if (r) return r;
@@ -859,8 +840,7 @@ static int renumber_one_record(struct index_record *mp,
     if (mp->system_flags & FLAG_EXPUNGED)
 	return 0;
 
-    if (kaction)
-	dlist_num(kaction, "RENUMBER", mp->uid);
+    dlist_num(kaction, "RENUMBER", mp->uid);
 
     return 0;
 }
@@ -912,19 +892,15 @@ static const char *make_flags(struct mailbox *mailbox, struct index_record *reco
 static void log_record(const char *name, struct mailbox *mailbox,
 		       struct index_record *record)
 {
-    syslog(LOG_NOTICE, "SYNCNOTICE: %s uid:%u modseq:" MODSEQ_FMT " "
-	  "last_updated:%lu internaldate:%lu flags:(%s)",
-	   name, record->uid, record->modseq,
-	   record->last_updated, record->internaldate,
-	   make_flags(mailbox, record));
+    syslog(LOG_ERR, "%s uid:%u modseq:" MODSEQ_FMT " last_updated:%lu internaldate:%lu flags:(%s)", name,
+	   record->uid, record->modseq, record->last_updated, record->internaldate, make_flags(mailbox, record));
 }
 
 static void log_mismatch(const char *reason, struct mailbox *mailbox,
 			 struct index_record *mp,
 			 struct index_record *rp)
 {
-    syslog(LOG_NOTICE, "SYNCNOTICE: record mismatch with replica: %s %s",
-	   mailbox->name, reason);
+    syslog(LOG_ERR, "RECORD MISMATCH WITH REPLICA: %s", reason);
     log_record("master", mailbox, mp);
     log_record("replica", mailbox, rp);
 }
@@ -938,22 +914,30 @@ static int compare_one_record(struct mailbox *mailbox,
     int i;
     int r;
 
-    /* are there any differences? */
+    /* if the GUIDs don't match, then treat as two 
+     * un-matched records :) */
+    if (!message_guid_compare(&mp->guid, &rp->guid)) {
+    	if (!(rp->system_flags & FLAG_EXPUNGED))
+	    dlist_num(kaction, "COPYBACK", rp->uid);
+    	if (!(mp->system_flags & FLAG_EXPUNGED))
+	    dlist_num(kaction, "RENUMBER", mp->uid);
+	return 0;
+    }
+
+    /* UIDs match, GUIDs match: look for differences in 
+     * everything else that's part of sync_crc! */
+
     if (mp->modseq != rp->modseq)
 	diff = 1;
-    else if (mp->last_updated != rp->last_updated)
+    if (mp->last_updated != rp->last_updated)
 	diff = 1;
-    else if (mp->internaldate != rp->internaldate)
+    if (mp->internaldate != rp->internaldate)
 	diff = 1;
-    else if (mp->system_flags != rp->system_flags)
+    if (mp->system_flags != rp->system_flags)
 	diff = 1;
-    else if (!message_guid_equal(&mp->guid, &rp->guid))
-	diff = 1;
-    else {
-	for (i = 0; i < MAX_USER_FLAGS/32; i++) {
-	    if (mp->user_flags[i] != rp->user_flags[i])
-		diff = 1;
-	}
+    for (i = 0; i < MAX_USER_FLAGS/32; i++) {
+	if (mp->user_flags[i] != rp->user_flags[i])
+	    diff = 1;
     }
 
     /* if differences we'll have to rewrite to bump the modseq
@@ -961,57 +945,46 @@ static int compare_one_record(struct mailbox *mailbox,
     if (diff) {
 	/* interesting case - expunged locally */
 	if (mp->system_flags & FLAG_EXPUNGED) {
-	    /* if expunged, fall through - the rewrite will lift
-	     * the modseq to force the change to stick */
+	    /* if the remote record is MORE recent, we
+	     * probably want to keep it */
+	    if (rp->modseq > mp->modseq ||
+		rp->last_updated > mp->last_updated)
+		return copyback_one_record(mailbox, rp, kaction);
+	    /* otherwise fall through - the modseq update
+	     * will cause it to expunge */
 	}
+	/* evil - expunged remotely, NOT locally */
 	else if (rp->system_flags & FLAG_EXPUNGED) {
-	    /* mark expunged - rewrite will cause both sides to agree
-	     * again */
-	    mp->system_flags |= FLAG_EXPUNGED;
+	    /* is the replica "newer"? */
+	    if (rp->modseq > mp->modseq ||
+		rp->last_updated > mp->last_updated) {
+		syslog(LOG_ERR, "recent expunged on replica %s:%u, expunging locally",
+		       mailbox->name, mp->uid);
+		mp->system_flags |= FLAG_EXPUNGED;
+	    }
+	    else {
+		/* will have to move the local record */
+		return renumber_one_record(mp, kaction);
+	    }
 	}
 
 	/* general case */
 	else {
-	    if (!message_guid_equal(&mp->guid, &rp->guid)) {
-		char *mguid = xstrdup(message_guid_encode(&mp->guid));
-		char *rguid = xstrdup(message_guid_encode(&rp->guid));
-		syslog(LOG_ERR, "SYNCERROR: guid mismatch %s %u (%s %s)",
-		       mailbox->name, mp->uid, rguid, mguid);
-		free(rguid);
-		free(mguid);
-		/* we will need to renumber both ends to get in sync */
-
-		/* ORDERING - always lower GUID first */
-		if (message_guid_cmp(&mp->guid, &rp->guid) < 0) {
-		    r = copyback_one_record(mailbox, rp, kaction);
-		    if (!r) r = renumber_one_record(mp, kaction);
-		}
-		else {
-		    r = renumber_one_record(mp, kaction);
-		    if (!r) r = copyback_one_record(mailbox, rp, kaction);
-		}
-
-		return r;
-	    }
-
 	    /* is the replica "newer"? */
-	    if (rp->modseq > mp->modseq &&
-		rp->last_updated >= mp->last_updated) {
-		/* then copy all the flag data over from the replica */
+	    if (rp->modseq > mp->modseq ||
+		rp->last_updated > mp->last_updated) {
+		log_mismatch("more recent on replica", mailbox, mp, rp);
 		mp->system_flags = rp->system_flags;
 		for (i = 0; i < MAX_USER_FLAGS/32; i++) 
 		    mp->user_flags[i] = rp->user_flags[i];
-
-		log_mismatch("more recent on replica", mailbox, mp, rp);
+		mp->internaldate = rp->internaldate;
+		/* no point copying modseq, it will be updated regardless */
 	    }
 	    else {
 		log_mismatch("more recent on master", mailbox, mp, rp);
 	    }
 	}
 
-	/* are we making changes yet? */
-	if (!kaction) return 0;
-
 	/* this will bump the modseq and force a resync either way :) */
 	r = mailbox_rewrite_index_record(mailbox, mp);
 	if (r) return r;
@@ -1020,114 +993,16 @@ static int compare_one_record(struct mailbox *mailbox,
     return 0;
 }
 
-
-static int mailbox_update_loop(struct mailbox *mailbox,
-			       struct dlist *ki,
-			       uint32_t last_uid,
-			       modseq_t highestmodseq,
-			       struct dlist *kaction)
-{
-    struct index_record mrecord;
-    struct index_record rrecord;
-    uint32_t recno = 1;
-    uint32_t old_num_records = mailbox->i.num_records;
-    int r;
-
-    /* while there are more records on either master OR replica,
-     * work out what to do with them */
-    while (ki || recno <= old_num_records) {
-	/* most common case - both a master AND a replica record exist */
-	if (ki && recno <= old_num_records) {
-	    r = mailbox_read_index_record(mailbox, recno, &mrecord);
-	    if (r) return r;
-	    r = parse_upload(ki, mailbox, &rrecord);
-	    if (r) return r;
-
-	    /* same UID - compare the records */
-	    if (rrecord.uid == mrecord.uid) {
-		r = compare_one_record(mailbox,
-				       &mrecord, &rrecord,
-				       kaction);
-		if (r) return r;
-		/* increment both */
-		recno++;
-		ki = ki->next;
-	    }
-	    else if (rrecord.uid > mrecord.uid) {
-		/* record only exists on the master */
-		if (!(mrecord.system_flags & FLAG_EXPUNGED)) {
-		    syslog(LOG_ERR, "SYNCERROR: only exists on master %s %u (%s)",
-			   mailbox->name, mrecord.uid,
-			   message_guid_encode(&mrecord.guid));
-		    r = renumber_one_record(&mrecord, kaction);
-		    if (r) return r;
-		}
-		/* only increment master */
-		recno++;
-	    }
-	    else {
-		/* record only exists on the replica */
-		if (!(rrecord.system_flags & FLAG_EXPUNGED)) {
-		    if (kaction)
-			syslog(LOG_ERR, "SYNCERROR: only exists on replica %s %u (%s)",
-			       mailbox->name, rrecord.uid,
-			       message_guid_encode(&rrecord.guid));
-		    r = copyback_one_record(mailbox, &rrecord, kaction);
-		    if (r) return r;
-		}
-		/* only increment replica */
-		ki = ki->next;
-	    }
-	}
-
-	/* no more replica records, but still master records */
-	else if (recno <= old_num_records) {
-	    r = mailbox_read_index_record(mailbox, recno, &mrecord);
-	    if (r) return r;
-	    /* if the replica has seen this UID, we need to renumber.
-	     * Otherwise it will replicate fine as-is */
-	    if (mrecord.uid <= last_uid) {
-		r = renumber_one_record(&mrecord, kaction);
-		if (r) return r;
-	    }
-	    else if (mrecord.modseq <= highestmodseq) {
-		if (kaction) {
-		    /* bump our modseq so we sync */
-		    syslog(LOG_NOTICE, "SYNCNOTICE: bumping modseq %s %u",
-			   mailbox->name, mrecord.uid);
-		    r = mailbox_rewrite_index_record(mailbox, &mrecord);
-		    if (r) return r;
-		}
-	    }
-	    recno++;
-	}
-
-	/* record only exists on the replica */
-	else {
-	    r = parse_upload(ki, mailbox, &rrecord);
-	    if (r) return r;
-
-	    if (kaction)
-		syslog(LOG_NOTICE, "SYNCNOTICE: only on replica %s %u",
-		       mailbox->name, rrecord.uid);
-
-	    /* going to need this one */
-	    r = copyback_one_record(mailbox, &rrecord, kaction);
-	    if (r) return r;
-
-	    ki = ki->next;
-	}
-    }
-
-    return 0;
-}
-
 static int mailbox_full_update(const char *mboxname)
 {
     const char *cmd = "FULLMAILBOX";
+    uint32_t recno;
+    unsigned old_num_records;
+    struct index_record mrecord, rrecord;
     struct mailbox *mailbox = NULL;
     int r;
     struct dlist *kin = NULL;
+    struct dlist *ki = NULL;
     struct dlist *kr = NULL;
     struct dlist *ka = NULL;
     struct dlist *kuids = NULL;
@@ -1188,11 +1063,12 @@ static int mailbox_full_update(const char *mboxname)
     r = mailbox_index_recalc(mailbox);
     if (r) goto done;
 
+    old_num_records = mailbox->i.num_records;
+
     /* if local UIDVALIDITY is lower, copy from remote, otherwise
      * remote will copy ours when we sync */
     if (mailbox->i.uidvalidity < uidvalidity) {
-	syslog(LOG_NOTICE, "SYNCNOTICE: uidvalidity higher on replica %s"
-	       ", updating %u => %u",
+	syslog(LOG_ERR, "%s uidvalidity higher on replica, updating %u => %u",
 	       mailbox->name, mailbox->i.uidvalidity, uidvalidity);
 	mailbox_index_dirty(mailbox);
 	mailbox->i.uidvalidity = uidvalidity;
@@ -1200,28 +1076,79 @@ static int mailbox_full_update(const char *mboxname)
 
     if (mailbox->i.highestmodseq < highestmodseq) {
 	mailbox_modseq_dirty(mailbox);
-	/* highestmodseq on replica is dirty - we must go to at least 
-	 * one higher! */
-	syslog(LOG_NOTICE, "SYNCNOTICE: highestmodseq higher on replica %s"
-	       ", updating " MODSEQ_FMT " => " MODSEQ_FMT,
-	       mailbox->name, mailbox->i.highestmodseq, highestmodseq+1);
+	/* highestmodseq on replica is dirty - we must go to at least one higher! */
 	mailbox->i.highestmodseq = highestmodseq+1;
     }
 
-    r = mailbox_update_loop(mailbox, kr->head, last_uid,
-			    highestmodseq, NULL);
-    if (r) {
-	syslog(LOG_ERR, "SYNCNOTICE: failed to prepare update for %s: %s",
-	       mailbox->name, error_message(r));
-	goto done;
-    }
-
-    /* OK - now we're committed to make changes! */
-
+    /* initialise the two loops */
     kaction = dlist_list(NULL, "ACTION");
-    r = mailbox_update_loop(mailbox, kr->head, last_uid,
-			    highestmodseq, kaction);
-    if (r) goto cleanup;
+    recno = 1;
+    ki = kr->head;
+
+    /* while there are more records on either master OR replica,
+     * work out what to do with them */
+    while (ki || recno <= old_num_records) {
+	/* most common case - both a master AND a replica record exist */
+	if (ki && recno <= old_num_records) {
+	    r = mailbox_read_index_record(mailbox, recno, &mrecord);
+	    if (r) goto done;
+	    r = parse_upload(ki, mailbox, &rrecord);
+	    if (r) goto done;
+
+	    /* same UID - compare the records */
+	    if (rrecord.uid == mrecord.uid) {
+		/* hasn't been changed already, check it */
+		if (mrecord.modseq <= highestmodseq) {
+		    r = compare_one_record(mailbox,
+					   &mrecord, &rrecord,
+					   kaction);
+		    if (r) goto done;
+		}
+		/* increment both */
+		recno++;
+		ki = ki->next;
+	    }
+	    else if (rrecord.uid > mrecord.uid) {
+		/* record only exists on the master */
+		r = renumber_one_record(&mrecord, kaction);
+		if (r) goto done;
+		/* only increment master */
+		recno++;
+	    }
+	    else {
+		/* record only exists on the replica */
+		r = copyback_one_record(mailbox, &rrecord, kaction);
+		if (r) goto done;
+		/* only increment replica */
+		ki = ki->next;
+	    }
+	}
+
+	/* no more replica records, but still master records */
+	else if (recno <= old_num_records) {
+	    r = mailbox_read_index_record(mailbox, recno, &mrecord);
+	    if (r) goto done;
+	    /* if the replica has seen this UID, we need to renumber.
+	     * Otherwise it will replicate fine as-is */
+	    if (mrecord.uid <= last_uid) {
+		r = renumber_one_record(&mrecord, kaction);
+		if (r) goto done;
+	    }
+	    recno++;
+	}
+
+	/* record only exists on the replica */
+	else {
+	    r = parse_upload(ki, mailbox, &rrecord);
+	    if (r) goto done;
+	    
+	    /* going to need this one */
+	    r = copyback_one_record(mailbox, &rrecord, kaction);
+	    if (r) goto done;
+
+	    ki = ki->next;
+	}
+    }
 
     /* if replica still has a higher last_uid, bump our local
      * number to match so future records don't clash */
@@ -1239,31 +1166,25 @@ static int mailbox_full_update(const char *mboxname)
 	}
 	else if (!strcmp(ka->name, "COPYBACK")) {
 	    r = copy_remote(mailbox, ka->nval, kr);
-	    if (r) goto cleanup;
+	    if (r) goto done;
 	    dlist_num(kuids, "UID", ka->nval);
 	}
 	else if (!strcmp(ka->name, "RENUMBER")) {
 	    r = copy_local(mailbox, ka->nval);
-	    if (r) goto cleanup;
+	    if (r) goto done;
 	}
     }
 
-    /* we still need to do the EXPUNGEs */
- cleanup:
-
-    /* close the mailbox before sending any expunges
+    /* commit and close the mailbox before sending any expunges
      * to avoid deadlocks */
+    r = mailbox_commit(mailbox);
+    if (r) goto done;
     mailbox_close(&mailbox);
 
     /* only send expunge if we have some UIDs to expunge */
     if (kuids->head) {
-	int r2;
 	sync_send_apply(kexpunge, sync_out);
-	r2 = sync_parse_response("EXPUNGE", sync_in, NULL);
-	if (r2) {
-	    syslog(LOG_ERR, "SYNCERROR: failed to expunge in cleanup %s",
-		   mboxname);
-	}
+	r = sync_parse_response("EXPUNGE", sync_in, NULL);
     }
 
 done:
@@ -1306,10 +1227,7 @@ static int update_mailbox_once(struct sync_folder *local,
     r = mailbox_open_irl(local->name, &mailbox);
     if (r == IMAP_MAILBOX_NONEXISTENT) {
 	/* been deleted in the meanwhile... */
-	if (remote)
-	    r = folder_delete(remote->name);
-	else
-	    r = 0;
+	r = folder_delete(remote->name);
 	goto done;
     }
     else if (r)
@@ -1430,8 +1348,8 @@ static int update_seen_work(const char *user, const char *uniqueid,
 static int do_seen(char *user, char *uniqueid)
 {
     int r = 0;
-    struct seen *seendb = NULL;
-    struct seendata sd = SEENDATA_INITIALIZER;
+    struct seen *seendb;
+    struct seendata sd;
 
     if (verbose) 
         printf("SEEN %s %s\n", user, uniqueid);
@@ -1444,10 +1362,14 @@ static int do_seen(char *user, char *uniqueid)
     if (r) return 0;
 
     r = seen_read(seendb, uniqueid, &sd);
+    if (r) {
+	seen_close(seendb);
+	return 0;
+    }
 
-    if (!r) r = update_seen_work(user, uniqueid, &sd);
+    r = update_seen_work(user, uniqueid, &sd);
 
-    seen_close(&seendb);
+    seen_close(seendb);
     seen_freedata(&sd);
 
     return r;
@@ -1598,10 +1520,7 @@ int do_folders(struct sync_name_list *mboxname_list,
 
     r = reserve_messages(mboxname_list, master_folders,
 			 replica_folders, reserve_guids);
-    if (r) {
-	syslog(LOG_ERR, "reserve messages: failed: %s", error_message(r));
-	goto bail;
-    }
+    if (r) goto bail;
 
     /* Tag folders on server which still exist on the client. Anything
      * on the server which remains untagged can be deleted immediately */
@@ -1620,11 +1539,7 @@ int do_folders(struct sync_name_list *mboxname_list,
     for (rfolder = replica_folders->head; rfolder; rfolder = rfolder->next) {
 	if (rfolder->mark) continue;
 	r = folder_delete(rfolder->name);
-	if (r) {
-	    syslog(LOG_ERR, "folder_delete(): failed: %s '%s'", 
-		   rfolder->name, error_message(r));
-	    goto bail;
-	}
+	if (r) goto bail;
     }
 
     /* Need to rename folders in an order which avoids dependancy conflicts
@@ -1639,11 +1554,8 @@ int do_folders(struct sync_name_list *mboxname_list,
 	for (item = rename_folders->head; item; item = item->next) {
 	    if (item->done) continue;
 
-	    /* don't skip rename to different partition */
-	    if (strcmp(item->oldname, item->newname)) {
-		item2 = sync_rename_lookup(rename_folders, item->newname);
-		if (item2 && !item2->done) continue;
-	    }
+	    item2 = sync_rename_lookup(rename_folders, item->newname);
+	    if (item2 && !item2->done) continue;
 
 	    /* Found unprocessed item which should rename cleanly */
 	    r = folder_rename(item->oldname, item->newname, item->part);
@@ -1833,23 +1745,32 @@ int do_user_main(char *user, struct sync_folder_list *replica_folders,
 
 int do_user_sub(const char *userid, struct sync_name_list *replica_subs)
 {
+    char buf[MAX_MAILBOX_BUFFER];
     struct sync_name_list *master_subs = sync_name_list_create();
     struct sync_name *msubs, *rsubs;
-    int r = 0;
+    int r;
 
     /* Includes subsiduary nodes automatically */
-    r = mboxlist_allsubs(userid, addmbox_sub, master_subs);
-    if (r) goto bail;
+    r = (sync_namespace.mboxlist_findsub)(&sync_namespace, "*", 1,
+                                          userid, NULL, addmbox_sub,
+                                          master_subs, 1);
+    if (r) {
+	syslog(LOG_ERR, "IOERROR: %s", error_message(r));
+	goto bail;
+    }
 
     /* add any folders that need adding, and mark any which
      * still exist */
     for (msubs = master_subs->head; msubs; msubs = msubs->next) {
-	rsubs = sync_name_lookup(replica_subs, msubs->name);
+	r = (sync_namespace.mboxname_tointernal)(&sync_namespace, msubs->name,
+						 userid, buf);
+	if (r) continue; /* just ignore invalid subs */
+	rsubs = sync_name_lookup(replica_subs, buf);
 	if (rsubs) {
 	    rsubs->mark = 1;
 	    continue;
 	}
-	r = set_sub(userid, msubs->name, 1);
+	r = set_sub(userid, buf, 1);
 	if (r) goto bail;
     }
 
@@ -1863,7 +1784,7 @@ int do_user_sub(const char *userid, struct sync_name_list *replica_subs)
 
  bail:
     sync_name_list_free(&master_subs);
-    return r;
+    return(r);
 }
 
 static int get_seen(const char *uniqueid, struct seendata *sd, void *rock)
@@ -1880,7 +1801,7 @@ static int do_user_seen(char *user, struct sync_seen_list *replica_seen)
 {
     int r;
     struct sync_seen *mseen, *rseen;
-    struct seen *seendb = NULL;
+    struct seen *seendb;
     struct sync_seen_list *list;
 
     /* silently ignore errors */
@@ -1890,7 +1811,7 @@ static int do_user_seen(char *user, struct sync_seen_list *replica_seen)
     list = sync_seen_list_create();
 
     seen_foreach(seendb, get_seen, list);
-    seen_close(&seendb);
+    seen_close(seendb);
 
     for (mseen = list->head; mseen; mseen = mseen->next) {
 	rseen = sync_seen_list_lookup(replica_seen, mseen->uniqueid);
@@ -2428,11 +2349,12 @@ int do_daemon_work(const char *sync_log_file, const char *sync_shutdown_file,
 
     *restartp = RESTART_NONE;
 
-    /* Create a work log filename.  Use the PID so we can
-     * try to reprocess it if the sync fails */
+    /* Create a work log filename.  Use the parent PID so we can
+     * try to reprocess it if the child fails.
+     */
     work_file_name = xmalloc(strlen(sync_log_file)+20);
     snprintf(work_file_name, strlen(sync_log_file)+20,
-             "%s-%d", sync_log_file, getpid());
+             "%s-%d", sync_log_file, getppid());
 
     session_start = time(NULL);
 
@@ -2454,7 +2376,8 @@ int do_daemon_work(const char *sync_log_file, const char *sync_shutdown_file,
             break;
         }
 
-        if (stat(work_file_name, &sbuf) == 0) {
+        if ((stat(work_file_name, &sbuf) == 0) &&
+	    (sbuf.st_mtime - single_start < 3600)) {
 	    /* Existing work log file from our parent < 1 hour old */
 	    /* XXX  Is 60 minutes a resonable timeframe? */
 	    syslog(LOG_NOTICE,
@@ -2616,43 +2539,39 @@ void replica_connect(const char *channel)
 		syslog(LOG_ERR, "unable to setsocketopt(TCP_NODELAY): %m");
 	    }
 
-	    /* turn on TCP keepalive if set */
-	    if (config_getswitch(IMAPOPT_TCP_KEEPALIVE)) {
+            /* turn on TCP keepalive if set */
+            if (config_getswitch(IMAPOPT_TCP_KEEPALIVE)) {
 		int r;
-		int optval = 1;
-		socklen_t optlen = sizeof(optval);
-		struct protoent *proto = getprotobyname("TCP");
+                int optval = 1;
+                socklen_t optlen = sizeof(optval);
 
-		r = setsockopt(sync_backend->sock, SOL_SOCKET, SO_KEEPALIVE, &optval, optlen);
-		if (r < 0) {
-		    syslog(LOG_ERR, "unable to setsocketopt(SO_KEEPALIVE): %m");
-		}
+                r = setsockopt(sync_backend->sock, SOL_SOCKET, SO_KEEPALIVE, &optval, optlen);
+                if (r < 0) {
+                    syslog(LOG_ERR, "unable to setsocketopt(SO_KEEPALIVE): %m");
+                }
 #ifdef TCP_KEEPCNT
-		optval = config_getint(IMAPOPT_TCP_KEEPALIVE_CNT);
-		if (optval) {
-		    r = setsockopt(sync_backend->sock, proto->p_proto, TCP_KEEPCNT, &optval, optlen);
-		    if (r < 0) {
-			syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPCNT): %m");
-		    }
-		}
+                if (config_getint(IMAPOPT_TCP_KEEPALIVE_CNT)) {
+                    r = setsockopt(sync_backend->sock, SOL_TCP, TCP_KEEPCNT, &optval, optlen);
+                    if (r < 0) {
+                        syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPCNT): %m");
+                    }
+                }
 #endif
 #ifdef TCP_KEEPIDLE
-		optval = config_getint(IMAPOPT_TCP_KEEPALIVE_IDLE);
-		if (optval) {
-		    r = setsockopt(sync_backend->sock, proto->p_proto, TCP_KEEPIDLE, &optval, optlen);
-		    if (r < 0) {
-			syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPIDLE): %m");
-		    }
-		}
+                if (config_getint(IMAPOPT_TCP_KEEPALIVE_IDLE)) {
+                    r = setsockopt(sync_backend->sock, SOL_TCP, TCP_KEEPIDLE, &optval, optlen);
+                    if (r < 0) {
+                        syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPIDLE): %m");
+                    }
+                }
 #endif
 #ifdef TCP_KEEPINTVL
-		optval = config_getint(IMAPOPT_TCP_KEEPALIVE_INTVL);
-		if (optval) {
-		    r = setsockopt(sync_backend->sock, proto->p_proto, TCP_KEEPINTVL, &optval, optlen);
-		    if (r < 0) {
-			syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPINTVL): %m");
-		    }
-		}
+                if (config_getint(IMAPOPT_TCP_KEEPALIVE_INTVL)) {
+                    r = setsockopt(sync_backend->sock, SOL_TCP, TCP_KEEPINTVL, &optval, optlen);
+                    if (r < 0) {
+                        syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPINTVL): %m");
+                    }
+                }
 #endif
 	    }
 	} else {
@@ -2679,11 +2598,6 @@ void replica_connect(const char *channel)
     /* links to sockets */
     sync_in = sync_backend->in;
     sync_out = sync_backend->out;
-
-    /* Force use of LITERAL+ so we don't need two way communications */
-    prot_setisclient(sync_in, 1);
-    prot_setisclient(sync_out, 1);
-
 }
 
 static void replica_disconnect()
@@ -2695,30 +2609,79 @@ void do_daemon(const char *sync_log_file, const char *sync_shutdown_file,
 	       const char *channel, unsigned long timeout, unsigned long min_delta)
 {
     int r = 0;
-    int restart = 1;
+    pid_t pid;
+    int status;
+    int restart;
+
+    if (!foreground) {
+	/* fork a child so we can release from master */
+	if ((pid=fork()) < 0) fatal("fork failed", EC_SOFTWARE);
+
+	if (pid != 0) { /* parent */
+	    cyrus_done();
+	    exit(0);
+	}
+	/* child */
+    }
+
+    if (foreground || timeout == 0) {
+	replica_connect(channel);
+        do_daemon_work(sync_log_file, sync_shutdown_file,
+                       timeout, min_delta, &restart);
+	replica_disconnect();
+        return;
+    }
 
     signal(SIGPIPE, SIG_IGN); /* don't fail on server disconnects */
 
-    while (restart) {
-	replica_connect(channel);
-	r = do_daemon_work(sync_log_file, sync_shutdown_file,
-			   timeout, min_delta, &restart);
-	if (r) {
-	    /* See if we're still connected to the server.
-	     * If we are, we had some type of error, so we exit.
-	     * Otherwise, try reconnecting.
-	     */
-	    if (!backend_ping(sync_backend)) restart = 1;
-	}
-	replica_disconnect();
+    do {
+	/* fork a child so we can RESTART (flush memory) */
+        if ((pid=fork()) < 0) fatal("fork failed", EC_SOFTWARE);
+
+        if (pid == 0) { /* child */
+	    replica_connect(channel);
+
+	    r = do_daemon_work(sync_log_file, sync_shutdown_file,
+			       timeout, min_delta, &restart);
+
+	    if (r) {
+		/* See if we're still connected to the server.
+		 * If we are, we had some type of error, so we exit.
+		 * Otherwise, try reconnecting.
+		 */
+		if (!backend_ping(sync_backend)) _exit(1);
+else 
+		syslog(LOG_WARNING, "Lost connection to server. Reconnecting");
+		restart = 1;
+	    }
+
+	    replica_disconnect();
+
+	    if (restart) _exit(EX_TEMPFAIL);
+	    else _exit(0);
+        }
+
+	/* parent */
+        if (waitpid(pid, &status, 0) < 0) fatal("waitpid failed", EC_SOFTWARE);
+
+    } while (WIFEXITED(status) && (WEXITSTATUS(status) == EX_TEMPFAIL));
+
+    if (WIFEXITED(status)) {
+	syslog(LOG_ERR, "process %d exited, status %d\n", pid, 
+	       WEXITSTATUS(status));
+    }
+    if (WIFSIGNALED(status)) {
+	syslog(LOG_ERR, 
+	       "process %d exited, signaled to death by %d\n",
+	       pid, WTERMSIG(status));
     }
 }
 
 /* ====================================================================== */
 
 static struct sasl_callback mysasl_cb[] = {
-    { SASL_CB_GETOPT, (mysasl_cb_ft *) &mysasl_config, NULL },
-    { SASL_CB_CANON_USER, (mysasl_cb_ft *) &mysasl_canon_user, NULL },
+    { SASL_CB_GETOPT, &mysasl_config, NULL },
+    { SASL_CB_CANON_USER, &mysasl_canon_user, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
@@ -2804,11 +2767,9 @@ int main(int argc, char **argv)
             min_delta = atoi(optarg);
             break;
 
-        case 'r':
-	    background = 1;
-	    /* fallthrough */
-
         case 'R':
+	    foreground = 1;
+        case 'r':
 	    if (mode != MODE_UNKNOWN)
 		fatal("Mutually exclusive options defined", EC_USAGE);
             mode = MODE_REPEAT;
@@ -2848,20 +2809,6 @@ int main(int argc, char **argv)
     if (mode == MODE_UNKNOWN)
         fatal("No replication mode specified", EC_USAGE);
 
-    /* fork if required */
-    if (background && !input_filename) {
-	int pid = fork();
-
-	if (pid == -1) {
-	    perror("fork");
-	    exit(1);
-	}
-
-	if (pid != 0) { /* parent */
-	    exit(0);
-	}
-    }
-
     cyrus_init(alt_config, "sync_client", 0);
 
     /* get the server name if not specified */
