@@ -47,9 +47,6 @@
 
 #include <config.h>
 
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -115,7 +112,7 @@ extern char *optarg;
 extern int opterr;
 
 /* for config.c */
-const int config_need_data = CONFIG_NEED_PARTITION_DATA;
+const int config_need_data = 0;
 
 static sasl_ssf_t extprops_ssf = 0;
 
@@ -174,9 +171,9 @@ static struct proxy_context sync_proxyctx = {
 };
 
 static struct sasl_callback mysasl_cb[] = {
-    { SASL_CB_GETOPT, (mysasl_cb_ft *) &mysasl_config, NULL },
-    { SASL_CB_PROXY_POLICY, (mysasl_cb_ft *) &mysasl_proxy_policy, (void*) &sync_proxyctx },
-    { SASL_CB_CANON_USER, (mysasl_cb_ft *) &mysasl_canon_user, NULL },
+    { SASL_CB_GETOPT, &mysasl_config, NULL },
+    { SASL_CB_PROXY_POLICY, &mysasl_proxy_policy, (void*) &sync_proxyctx },
+    { SASL_CB_CANON_USER, &mysasl_canon_user, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
@@ -349,10 +346,6 @@ int service_main(int argc __attribute__((unused)),
     sync_in = prot_new(0, 0);
     sync_out = prot_new(1, 1);
 
-    /* Force use of LITERAL+ so we don't need two way communications */
-    prot_setisclient(sync_in, 1);
-    prot_setisclient(sync_out, 1);
-
     /* Find out name of client host */
     salen = sizeof(sync_remoteaddr);
     if (getpeername(0, (struct sockaddr *)&sync_remoteaddr, &salen) == 0 &&
@@ -708,10 +701,6 @@ static void cmdloop(void)
 	    if (!strcmp(cmd.s, "Starttls") && tls_enabled()) {
 		if (c == '\r') c = prot_getc(sync_in);
 		if (c != '\n') goto extraargs;
-
-		/* XXX  discard any input pipelined after STARTTLS */
-		prot_flush(sync_in);
-
 		/* if we've already done SASL fail */
 		if (sync_userid != NULL) {
 		    prot_printf(sync_out, 
@@ -767,7 +756,6 @@ static void cmd_authenticate(char *mech, char *resp)
     sasl_ssf_t ssf;
     char *ssfmsg = NULL;
     const void *val;
-    int failedloginpause;
 
     if (sync_userid) {
 	prot_printf(sync_out, "BAD Already authenticated\r\n");
@@ -799,10 +787,7 @@ static void cmd_authenticate(char *mech, char *resp)
 	    syslog(LOG_NOTICE, "badlogin: %s %s [%s]",
 		   sync_clienthost, mech, sasl_errdetail(sync_saslconn));
 
-	    failedloginpause = config_getint(IMAPOPT_FAILEDLOGINPAUSE);
-	    if (failedloginpause != 0) {
-	        sleep(failedloginpause);
-	    }
+	    sleep(3);
 
 	    if (errorstring) {
 		prot_printf(sync_out, "NO %s\r\n", errorstring);
@@ -979,48 +964,6 @@ static void cmd_compress(char *alg)
 }
 #endif
 
-/* ====================================================================== */
-
-/* partition_list is simple linked list of names used by cmd_restart */
-
-struct partition_list {
-    struct partition_list *next;
-    char *name;
-};
-
-static struct partition_list *
-partition_list_add(char *name, struct partition_list *pl)
-{
-    struct partition_list *p;
-
-    /* Is name already on list? */
-    for (p=pl; p; p = p->next) {
-        if (!strcmp(p->name, name))
-            return(pl);
-    }
-
-    /* Add entry to start of list and return new list */
-    p = xzmalloc(sizeof(struct partition_list));
-    p->next = pl;
-    p->name = xstrdup(name);
-
-    return(p);
-}
-
-static void
-partition_list_free(struct partition_list *current)
-{
-    while (current) {
-        struct partition_list *next = current->next;
-
-        free(current->name);
-        free(current);
-
-        current = next;
-    }
-    
-}
-
 static void cmd_restart(struct sync_reserve_list **reserve_listp, int re_alloc)
 {
     struct sync_reserve *res;
@@ -1028,27 +971,14 @@ static void cmd_restart(struct sync_reserve_list **reserve_listp, int re_alloc)
     struct sync_msgid *msg;
     const char *fname;
     int hash_size = l->hash_size;
-    struct partition_list *p, *pl = NULL;
 
     for (res = l->head; res; res = res->next) {
 	for (msg = res->list->head; msg; msg = msg->next) {
-            pl = partition_list_add(res->part, pl);
-
 	    fname = dlist_reserve_path(res->part, &msg->guid);
 	    unlink(fname);
 	}
     }
     sync_reserve_list_free(reserve_listp);
-        
-    /* Remove all <partition>/sync./<pid> directories referred to above */
-    for (p=pl; p ; p = p->next) {
-        static char buf[MAX_MAILBOX_PATH];
-
-        snprintf(buf, MAX_MAILBOX_PATH, "%s/sync./%lu", 
-                 config_partitiondir(p->name), (unsigned long)getpid());
-        rmdir(buf);
-    }
-    partition_list_free(pl);
 
     if (re_alloc)
 	*reserve_listp = sync_reserve_list_create(hash_size);
@@ -1099,7 +1029,7 @@ void reserve_folder(const char *part, const char *mboxname,
 		   mailbox_msg_path);
 	    continue;
 	}
-	if (!message_guid_equal(&record.guid, &record2.guid)) {
+	if (!message_guid_compare(&record.guid, &record2.guid)) {
 	    syslog(LOG_ERR, "IOERROR: GUID mismatch on parse for %s",
 		   mailbox_msg_path);
 	    continue;
@@ -1215,108 +1145,8 @@ static int do_quota(struct dlist *kin)
 
 /* ====================================================================== */
 
-static int mailbox_compare_update(struct mailbox *mailbox,
-				  struct dlist *kr, int doupdate)
-{
-    struct index_record mrecord;
-    struct index_record rrecord;
-    uint32_t recno = 1;
-    struct dlist *ki;
-    int r;
-    int i;
-
-    rrecord.uid = 0;
-    for (ki = kr->head; ki; ki = ki->next) {
-	r = parse_upload(ki, mailbox, &mrecord);
-	if (r) {
-	    syslog(LOG_ERR, "Failed to parse uploaded record"); 
-	    return r;
-	}
-
-	while (rrecord.uid < mrecord.uid) {
-	    /* hit the end?  Magic marker */
-	    if (recno > mailbox->i.num_records) {
-		rrecord.uid = UINT32_MAX;
-		break;
-	    }
-
-	    /* read another record */
-	    r = mailbox_read_index_record(mailbox, recno, &rrecord);
-	    if (r) {
-		syslog(LOG_ERR, "Failed to read record %s %d",
-		       mailbox->name, recno);
-		return r;
-	    }
-	    recno++;
-	}
-
-	/* found a match, check for updates */
-	if (rrecord.uid == mrecord.uid) {
-	    /* GUID mismatch on a non-expunged record is an error straight away */
-	    if (!(mrecord.system_flags & FLAG_EXPUNGED)) {
-		if (!message_guid_equal(&mrecord.guid, &rrecord.guid)) {
-		    syslog(LOG_ERR, "SYNCERROR: guid mismatch %s %u",
-			   mailbox->name, mrecord.uid);
-		    return IMAP_MAILBOX_CRC;
-		}
-		if (rrecord.system_flags & FLAG_EXPUNGED) {
-		    syslog(LOG_ERR, "SYNCERROR: expunged on replica %s %u",
-			   mailbox->name, mrecord.uid);
-		    return IMAP_MAILBOX_CRC;
-		}
-	    }
-	    /* higher modseq on the replica is an error */
-	    if (rrecord.modseq > mrecord.modseq) {
-		syslog(LOG_ERR, "SYNCERROR: higher modseq on replica %s %u",
-		       mailbox->name, mrecord.uid);
-		return IMAP_MAILBOX_CRC;
-	    }
-
-	    /* skip out on the first pass */
-	    if (!doupdate) continue;
-
-	    rrecord.modseq = mrecord.modseq;
-	    rrecord.last_updated = mrecord.last_updated;
-	    rrecord.internaldate = mrecord.internaldate;
-	    rrecord.system_flags = (mrecord.system_flags & ~FLAG_UNLINKED) |
-				   (rrecord.system_flags & FLAG_UNLINKED);
-	    for (i = 0; i < MAX_USER_FLAGS/32; i++)
-		rrecord.user_flags[i] = mrecord.user_flags[i];
-	    rrecord.silent = 1;
-	    r = mailbox_rewrite_index_record(mailbox, &rrecord);
-	    if (r) {
-		syslog(LOG_ERR, "IOERROR: failed to rewrite record %s %d",
-		       mailbox->name, recno);
-		return r;
-	    }
-	}
-
-	/* not found and less than LAST_UID, bogus */
-	else if (mrecord.uid <= mailbox->i.last_uid) {
-	    /* Expunged, just skip it */
-	    if (!(mrecord.system_flags & FLAG_EXPUNGED))
-		return IMAP_MAILBOX_CRC;
-	}
-
-	/* after LAST_UID, it's an append, that's OK */
-	else {
-	    /* skip out on the first pass */
-	    if (!doupdate) continue;
-
-	    mrecord.silent = 1;
-	    r = sync_append_copyfile(mailbox, &mrecord);
-	    if (r) {
-		syslog(LOG_ERR, "IOERROR: failed to append file %s %d",
-		       mailbox->name, recno);
-		return r;
-	    }
-	}
-    }
-
-    return 0;
-}
-
-static int do_mailbox(struct dlist *kin)
+static int do_mailbox(struct dlist *kin,
+		      struct sync_reserve_list *reserved_list)
 {
     /* fields from the request */
     const char *uniqueid;
@@ -1336,10 +1166,20 @@ static int do_mailbox(struct dlist *kin)
     uint32_t options;
 
     struct mailbox *mailbox = NULL;
+    struct index_record mrecord, rrecord;
+    struct sync_msgid_list *part_list;
+    unsigned old_num_records;
     uint32_t newcrc;
     struct dlist *kr;
+    struct dlist *ki;
+    int found;
+    uint32_t recno;
     int r;
 
+    /* probably should be moved */
+    int i;
+
+
     if (!dlist_getatom(kin, "UNIQUEID", &uniqueid))
 	return IMAP_PROTOCOL_BAD_PARAMETERS;
     if (!dlist_getatom(kin, "PARTITION", &partition))
@@ -1375,8 +1215,8 @@ static int do_mailbox(struct dlist *kin)
     if (r == IMAP_MAILBOX_NONEXISTENT) {
 	r = mboxlist_createsync(mboxname, 0, partition,
 				sync_userid, sync_authstate,
-				options, uidvalidity, acl,
-				uniqueid, &mailbox);
+				options, uidvalidity, acl, uniqueid);
+	if (!r) r = mailbox_open_iwl(mboxname, &mailbox);
     }
     if (r) {
 	syslog(LOG_ERR, "Failed to open mailbox %s to update", mboxname);
@@ -1398,15 +1238,8 @@ static int do_mailbox(struct dlist *kin)
 	return IMAP_MAILBOX_CRC;
     }
 
-    if (last_uid < mailbox->i.last_uid) {
-	syslog(LOG_ERR, "higher last_uid on replica %s - %u < %u",
-	       mboxname, last_uid, mailbox->i.last_uid);
-	mailbox_close(&mailbox);
-	return IMAP_MAILBOX_CRC;
-    }
-
     if (strcmp(mailbox->acl, acl)) {
-	mailbox_set_acl(mailbox, acl, 0);
+	mailbox_set_acl(mailbox, acl);
 	r = mboxlist_sync_setacls(mboxname, acl);
 	if (r) {
 	    mailbox_close(&mailbox);
@@ -1414,40 +1247,102 @@ static int do_mailbox(struct dlist *kin)
 	}
     }
 
-    r = mailbox_compare_update(mailbox, kr, 0);
-    if (r) {
-	mailbox_close(&mailbox);
-	return r;
-    }
-
     /* now we're committed to writing something no matter what happens! */
 
-    r = mailbox_compare_update(mailbox, kr, 1);
-    if (r) {
-	abort();
-	return r;
-    }
-
-    mailbox_index_dirty(mailbox);
-    assert(mailbox->i.last_uid <= last_uid);
-    mailbox->i.last_uid = last_uid;
-    mailbox->i.highestmodseq = highestmodseq;
-    mailbox->i.recentuid = recentuid;
-    mailbox->i.recenttime = recenttime;
-    mailbox->i.last_appenddate = last_appenddate;
-    mailbox->i.pop3_last_login = pop3_last_login;
-    /* mailbox->i.options = options; ... not really, there's unsyncable stuff in here */
-
     if (mailbox->i.uidvalidity < uidvalidity) {
 	syslog(LOG_ERR, "%s uidvalidity higher on master, updating %u => %u",
 	       mailbox->name, mailbox->i.uidvalidity, uidvalidity);
+	mailbox_index_dirty(mailbox);
 	mailbox->i.uidvalidity = uidvalidity;
     }
 
+    part_list = sync_reserve_partlist(reserved_list, mailbox->part);
+    old_num_records = mailbox->i.num_records;
+
+    rrecord.uid = 0;
+    recno = 1;
+    for (ki = kr->head; ki; ki = ki->next) {
+	r = parse_upload(ki, mailbox, &mrecord);
+	if (r) {
+	    syslog(LOG_ERR, "Failed to parse uploaded record"); 
+	    goto done;
+	}
+	found = 0;
+	while (!found && recno <= old_num_records) {
+	    r = mailbox_read_index_record(mailbox, recno, &rrecord);
+	    if (r) {
+		syslog(LOG_ERR, "Failed to read record %s %d",
+		       mboxname, recno);
+		goto done;
+	    }
+
+	    /* copy the interesting fields out */
+	    if (rrecord.uid == mrecord.uid) {
+		/* GUID mismatch on a non-expunged record is an error straight away */
+		if (!(mrecord.system_flags & FLAG_EXPUNGED) &&
+		    !message_guid_compare(&mrecord.guid, &rrecord.guid)) {
+		    r = IMAP_MAILBOX_CRC;
+		    goto done;
+		}
+		rrecord.modseq = mrecord.modseq;
+		rrecord.last_updated = mrecord.last_updated;
+		rrecord.internaldate = mrecord.internaldate;
+		rrecord.system_flags = (mrecord.system_flags & ~FLAG_UNLINKED) |
+				       (rrecord.system_flags & FLAG_UNLINKED);
+		for (i = 0; i < MAX_USER_FLAGS/32; i++)
+		    rrecord.user_flags[i] = mrecord.user_flags[i];
+		rrecord.silent = 1;
+		r = mailbox_rewrite_index_record(mailbox, &rrecord);
+		if (r) {
+		    syslog(LOG_ERR, "Failed to rewrite record %s %d",
+			   mboxname, recno);
+		    goto done;
+		}
+		found = 1;
+	    }
+	    recno++;
+	}
+	if (found) continue;
+	if (rrecord.uid >= mrecord.uid)
+	    continue; /* guess it got UNLINKED along the way... */
+
+	/* not found and less than LAST_UID, bogus */
+	if (mrecord.uid <= mailbox->i.last_uid) {
+	    /* Expunged, just skip it */
+	    if (mrecord.system_flags & FLAG_EXPUNGED)
+		continue;
+	    /* Otherwise we want to abort and fix up */
+	    r = IMAP_MAILBOX_CRC;
+	    goto done;
+	}
+
+	mrecord.silent = 1;
+	r = sync_append_copyfile(mailbox, &mrecord);
+	if (r) {
+	    syslog(LOG_ERR, "Failed to append file %s %d",
+		   mboxname, recno);
+	    goto done;
+	}
+    }
+
+ done:
+    if (!r) {
+	mailbox_index_dirty(mailbox);
+	mailbox->i.last_uid = last_uid;
+	mailbox->i.highestmodseq = highestmodseq;
+	mailbox->i.recentuid = recentuid;
+	mailbox->i.recenttime = recenttime;
+	mailbox->i.last_appenddate = last_appenddate;
+	mailbox->i.pop3_last_login = pop3_last_login;
+	mailbox->i.uidvalidity = uidvalidity;
+	/* mailbox->i.options = options; ... not really, there's unsyncable stuff in here */
+    }
+
     /* try re-calculating the CRC on mismatch... */
     if (mailbox->i.sync_crc != sync_crc) {
 	mailbox_index_recalc(mailbox);
     }
+    mailbox_commit(mailbox);
     newcrc = mailbox->i.sync_crc;
     mailbox_close(&mailbox);
 
@@ -1589,14 +1484,14 @@ static int print_seen(const char *uniqueid, struct seendata *sd,
 
 static int user_seen(const char *userid)
 {
-    struct seen *seendb = NULL;
+    struct seen *seendb;
 
     /* no SEEN DB is OK, just return */
     if (seen_open(userid, SEEN_SILENT, &seendb))
 	return 0;
 
     seen_foreach(seendb, print_seen, NULL);
-    seen_close(&seendb);
+    seen_close(seendb);
 
     return 0;
 }
@@ -1604,15 +1499,25 @@ static int user_seen(const char *userid)
 
 static int user_sub(const char *userid)
 {
-    struct sync_name_list *list = sync_name_list_create();
+    char buf[MAX_MAILBOX_BUFFER];
+    struct sync_name_list *list = NULL;
     struct sync_name *item;
     struct dlist *kl;
+    int r;
 
-    mboxlist_allsubs(userid, addmbox_sub, list);
+    list = sync_name_list_create();
+
+    ((*sync_namespacep).mboxlist_findsub)(sync_namespacep, "*",
+					  sync_userisadmin,
+					  (char *)userid, sync_authstate,
+					  addmbox_sub, list, 1);
 
     kl = dlist_list(NULL, "LSUB");
     for (item = list->head; item; item = item->next) {
-	dlist_atom(kl, "MBOXNAME", item->name);
+	r = (sync_namespacep->mboxname_tointernal)(sync_namespacep, item->name,
+						   userid, buf);
+        if (r) continue;
+	dlist_atom(kl, "MBOXNAME", buf);
     }
     if (kl->head)
 	sync_send_response(kl, sync_out);
@@ -1886,7 +1791,7 @@ static int do_unactivate_sieve(struct dlist *kin)
 static int do_seen(struct dlist *kin)
 {
     int r;
-    struct seen *seendb = NULL;
+    struct seen *seendb;
     struct seendata sd;
     const char *userid;
     const char *uniqueid;
@@ -1908,25 +1813,32 @@ static int do_seen(struct dlist *kin)
     if (r) return r;
 
     r = seen_write(seendb, uniqueid, &sd);
-    seen_close(&seendb);
+    seen_close(seendb);
 
     return r;
 }
 
 static int do_unuser(struct dlist *kin)
 {
-    struct sync_name_list *list = sync_name_list_create();
+    struct sync_name_list *list = NULL;
     struct sync_name *item;
+    char buf[MAX_MAILBOX_BUFFER];
     const char *userid = kin->sval;
-    char buf[MAX_MAILBOX_NAME];
     int r = 0;
 
     /* Nuke subscriptions */
-    mboxlist_allsubs(userid, addmbox_sub, list);
+    list = sync_name_list_create();
+    r = (sync_namespacep->mboxlist_findsub)(sync_namespacep, "*",
+					    sync_userisadmin,
+                                            (char *)userid, sync_authstate,
+					    addmbox_sub, (void *)list, 1);
+    if (r) goto fail;
 
     /* ignore failures here - the subs file gets deleted soon anyway */
     for (item = list->head; item; item = item->next) {
-        mboxlist_changesub(item->name, userid, sync_authstate, 0, 0);
+	r = (sync_namespacep->mboxname_tointernal)(sync_namespacep, item->name,
+						   userid, buf);
+        if (!r) r = mboxlist_changesub(buf, userid, sync_authstate, 0, 0);
     }
     sync_name_list_free(&list);
 
@@ -2036,7 +1948,7 @@ static int do_expunge(struct dlist *kin)
     struct mailbox *mailbox = NULL;
     struct index_record record;
     uint32_t recno;
-    int r = 0;
+    int r;
 
     if (!dlist_getatom(kin, "MBOXNAME", &mboxname))
 	return IMAP_PROTOCOL_BAD_PARAMETERS;
@@ -2046,7 +1958,7 @@ static int do_expunge(struct dlist *kin)
 	return IMAP_PROTOCOL_BAD_PARAMETERS;
 
     r = mailbox_open_iwl(mboxname, &mailbox);
-    if (r) goto done;
+    if (r) return r;
 
     /* don't want to expunge the wrong mailbox! */
     if (strcmp(mailbox->uniqueid, uniqueid)) {
@@ -2058,17 +1970,19 @@ static int do_expunge(struct dlist *kin)
 
     for (recno = 1; recno <= mailbox->i.num_records; recno++) {
 	r = mailbox_read_index_record(mailbox, recno, &record);
-	if (r) goto done;
+	if (r) return r;
 	if (record.system_flags & FLAG_EXPUNGED) continue;
 	while (ui && ui->nval < record.uid) ui = ui->next;
 	if (!ui) break; /* no point continuing */
 	if (record.uid == ui->nval) {
 	    record.system_flags |= FLAG_EXPUNGED;
-	    record.silent = 1; /* so the next sync will succeed */
+	    /* don't get the modseqs out of sync too! */
+	    record.silent = 1;
 	    r = mailbox_rewrite_index_record(mailbox, &record);
 	    if (r) goto done;
 	}
     }
+    r = mailbox_commit(mailbox);
 
 done:
     mailbox_close(&mailbox);
@@ -2139,7 +2053,7 @@ static void cmd_apply(struct dlist *kin, struct sync_reserve_list *reserve_list)
     else if (!strcmp(kin->name, "ANNOTATION"))
 	r = do_annotation(kin);
     else if (!strcmp(kin->name, "MAILBOX"))
-	r = do_mailbox(kin);
+	r = do_mailbox(kin, reserve_list);
     else if (!strcmp(kin->name, "QUOTA"))
 	r = do_quota(kin);
     else if (!strcmp(kin->name, "SEEN"))
