@@ -393,7 +393,6 @@ int user_renameacl(char *name, char *olduser, char *newuser)
     char *aclalloc;
 
     r = mboxlist_lookup(name, &mbentry, NULL);
-    if (r) return r;
 
     /* setacl re-calls mboxlist_lookup and will stomp on us */
     aclalloc = acl = xstrdup(mbentry.acl);
@@ -435,21 +434,25 @@ int user_copyquotaroot(char *oldname, char *newname)
     return r;
 }
 
+struct find_rock {
+    char *inboxname;
+    struct txn **tid;
+};
+
 static int find_p(void *rockp,
 		  const char *key, int keylen,
 		  const char *data __attribute__((unused)),
 		  int datalen __attribute__((unused)))
 {
-    char *inboxname = (char *)rockp;
-    int inboxlen = strlen(inboxname);
+    char *inboxname = ((struct find_rock *) rockp)->inboxname;
 
-    return (!strncmp(key, inboxname, inboxlen) &&
-	    (keylen == inboxlen || key[inboxlen] == '.'));
+    return (!strncmp(key, inboxname, strlen(inboxname)) &&
+	    (keylen == (int) strlen(inboxname) || key[strlen(inboxname)] == '.'));
 }
 
 static int find_cb(void *rockp __attribute__((unused)),
-		   const char *key, int keylen,
-		   const char *data __attribute__((unused)),
+		  const char *key, int keylen,
+		  const char *data __attribute__((unused)),
 		   int datalen __attribute__((unused)))
 {
     char *root;
@@ -466,6 +469,7 @@ int user_deletequotaroots(const char *user)
 {
     struct namespace namespace;
     char buf[MAX_MAILBOX_BUFFER], *inboxname = buf;
+    struct txn *tid = NULL;
     int r;
 
     /* set namespace */
@@ -478,10 +482,15 @@ int user_deletequotaroots(const char *user)
     }
 
     if (!r) {
+	struct find_rock frock = { NULL, NULL };
+	frock.inboxname = inboxname;
+	frock.tid = &tid;
 	r = config_quota_db->foreach(qdb, inboxname, strlen(inboxname),
-				     &find_p, &find_cb, inboxname, NULL);
+				     &find_p, &find_cb, &frock, &tid);
     }
 
+    if (!r) quota_commit(&tid);
+
     return r;
 }
 
