@@ -45,6 +45,36 @@
 #ifndef _CYRUS_VERSION_H
 #define _CYRUS_VERSION_H
 
+enum {
+    CAPA_PREAUTH = 0x1,
+    CAPA_POSTAUTH = 0x2
+};
+
+/* CAPABILITIES are defined here, not including TLS/SASL ones,
+   and those that are configurable */
+#define CAPA_PREAUTH_STRING "IMAP4rev1 LITERAL+ ID ENABLE"
+
+#ifdef HAVE_SSL
+#define CAPA_URLAUTH " URLAUTH URLAUTH=BINARY"
+#else
+#define CAPA_URLAUTH ""
+#endif
+
+#ifdef ENABLE_X_NETSCAPE_HACK
+#define CAPA_NETSCAPE " X-NETSCAPE"
+#else
+#define CAPA_NETSCAPE ""
+#endif
+
+#define CAPA_POSTAUTH_STRING " ACL RIGHTS=kxte QUOTA " \
+	"MAILBOX-REFERRALS NAMESPACE UIDPLUS " \
+	"NO_ATOMIC_RENAME UNSELECT " \
+	"CHILDREN MULTIAPPEND BINARY CATENATE CONDSTORE ESEARCH " \
+	"SORT SORT=MODSEQ THREAD=ORDEREDSUBJECT THREAD=REFERENCES " \
+	"ANNOTATEMORE LIST-EXTENDED WITHIN QRESYNC SCAN XLIST" \
+	CAPA_URLAUTH CAPA_NETSCAPE
+
+
 /* Values for ID processing */
 enum {
     MAXIDFAILED	= 3,
