@@ -83,20 +83,27 @@
 #include "hash.h"
 #include "imparse.h"
 #include "iptostring.h"
-#include "stristr.h"
 #include "util.h"
 #include "xmalloc.h"
 #include "xstrlcat.h"
 #include "xstrlcpy.h"
-#include "md5.h"
 
 #ifdef HAVE_SSL
 #include <openssl/ssl.h>
+#include <openssl/md5.h>
 
 static SSL_CTX *tls_ctx = NULL;
 static SSL *tls_conn = NULL;
 static SSL_SESSION *tls_sess = NULL;
 
+#else /* HAVE_SSL */
+#include <sasl/md5global.h>
+#include <sasl/md5.h>
+
+#define MD5_Init _sasl_MD5Init
+#define MD5_Update _sasl_MD5Update
+#define MD5_Final _sasl_MD5Final
+
 #endif /* HAVE_SSL */
 
 #define IMTEST_OK    0
@@ -178,8 +185,7 @@ struct capa_cmd_t {
     char *cmd;		/* capability command string (NULL = no capa cmd) */
     char *resp;		/* end of capability response */
     char *tls;		/* [OPTIONAL] TLS capability string */
-    char *login;	/* [OPTIONAL] plaintext login cmd capability string */
-    char *auth;		/* [OPTIONAL] AUTH (SASL) capability string */
+    char *auth;		/* [OPTIONAL] AUTH capability string */
     char *compress;	/* [OPTIONAL] COMPRESS capability string */
     char *(*parse_mechlist)(const char *str, struct protocol_t *prot);
 			/* [OPTIONAL] parse capability string,
@@ -224,17 +230,15 @@ struct protocol_t {
     char *protocol;	/* protocol service name */
     char *sprotocol;	/* SSL-wrapped service name (NULL = unsupported) */
     char *service;	/* SASL service name */
-    int login_enabled;	/* [OPTIONAL] login command on/off by default;
-			   toggled by capability string */
     struct banner_t banner;
     struct capa_cmd_t capa_cmd;
     struct tls_cmd_t tls_cmd;
     struct sasl_cmd_t sasl_cmd;
     struct compress_cmd_t compress_cmd;
     int (*do_auth)(struct sasl_cmd_t *sasl_cmd, void *rock,
-		   int login_enabled, char *mech, char *mechlist);
-			/* [OPTIONAL] perform protocol-specific authentication;
-			   based on rock, login_enabled, mech, mechlist */
+		   char *mech, char *mechlist);
+			/* [OPTIONAL] perform protocol-specific
+			   authentication; based on rock, mech, mechlist */
     struct logout_cmd_t logout_cmd;
 
     /* these 3 are used for maintaining connection state */
@@ -506,7 +510,7 @@ static int tls_init_clientengine(int verifydepth, char *var_tls_cert_file, char
 	return IMTEST_FAIL;
     }
     
-    tls_ctx = SSL_CTX_new(SSLv23_client_method());
+    tls_ctx = SSL_CTX_new(TLSv1_client_method());
     if (tls_ctx == NULL) {
 	return IMTEST_FAIL;
     };
@@ -652,7 +656,7 @@ static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
 int tls_start_clienttls(unsigned *layer, char **authid)
 {
     int     sts;
-    const SSL_CIPHER *cipher;
+    SSL_CIPHER *cipher;
     X509   *peer;
     
     if (verbose==1)
@@ -945,7 +949,7 @@ void interaction (int id, const char *challenge, const char *prompt,
     
     if (id==SASL_CB_PASS && !cmdline_password) {
 	printf("%s: ", prompt);
-	cur->str=strdup(cyrus_getpass(""));
+	cur->str=strdup(getpass(""));
 	*tlen=strlen(cur->str);
 	*tresult = cur->str;
 	return;
@@ -976,7 +980,7 @@ void interaction (int id, const char *challenge, const char *prompt,
 	}
 	printf("%s: ",prompt);
 	if (id==SASL_CB_NOECHOPROMPT) {
-	    strcpy(result, cyrus_getpass(""));
+	    strcpy(result, getpass(""));
 	} else {
 	    fgets(result, sizeof(result) - 1, stdin);
 	    c = strlen(result);
@@ -1204,20 +1208,6 @@ static void sigint_handler(int sig __attribute__((unused)))
     gotsigint = 1;
 }
 
-static int haveinput(struct protstream *s)
-{
-    /* Is something currently pending in our protstream's buffer? */
-#ifdef HAVE_SSL
-    if (s->cnt == 0 && s->tls_conn != NULL) {
-	/* Maybe there's data pending in the SSL buffer? */
-	int n = SSL_pending(s->tls_conn);
-	if (verbose) printf("SSL_pending=%d\n", n);
-	return n;
-    }
-#endif
-    return s->cnt;
-}
-
 /* This needs to support 3 modes:
  *
  * 1. Terminal Interface Only
@@ -1382,7 +1372,7 @@ static void interactive(struct protocol_t *protocol, char *filename)
 		    buf[count] = '\0';
 		    printf("%s", buf); 
 		}
-	    } while (haveinput(pin) > 0);
+	    } while (pin->cnt > 0);
 	} else if ((FD_ISSET(fd, &rset)) && (FD_ISSET(sock, &wset))
 		   && (donewritingfile == 0)) {
 	    /* This does input for both socket and file modes */
@@ -1481,20 +1471,14 @@ enum {
     AUTO_YES = 1
 };
 
-enum {
-    CAPA_LOGIN		= (1 << 0),
-    CAPA_STARTTLS	= (1 << 1),
-    CAPA_COMPRESS	= (1 << 2)
-};
-
 static char *ask_capability(struct protocol_t *prot,
-			    unsigned long *capabilities, int automatic)
+			    int *supports_starttls, int *supports_compress,
+			    int automatic)
 {
     char str[1024] = "";
     char *ret = NULL, *tmp, *resp;
     
-    /* default state of login command unless toggled by capabilities */
-    *capabilities = prot->login_enabled;
+    *supports_starttls = 0;
 
     resp = (automatic == AUTO_BANNER) ? prot->banner.resp : prot->capa_cmd.resp;
 
@@ -1509,34 +1493,31 @@ static char *ask_capability(struct protocol_t *prot,
     }
 
     do { /* look for the end of the capabilities */
+	if (ret) {
+	    free(ret);
+	    ret = NULL;
+	}
 	if (prot_fgets(str, sizeof(str), pin) == NULL) {
 	    if (!*str) imtest_fatal("prot layer failure");
 	    else break;
 	}
 	printf("S: %s", str);
 
-	/* check for login - toggles existing state */
-	if (prot->capa_cmd.login &&
-	    strstr(str, prot->capa_cmd.login) != NULL) {
-	    *capabilities ^= CAPA_LOGIN;
-	}
-	
 	/* check for starttls */
 	if (prot->capa_cmd.tls &&
 	    strstr(str, prot->capa_cmd.tls) != NULL) {
-	    *capabilities |= CAPA_STARTTLS;
+	    *supports_starttls = 1;
 	}
 	
 	/* check for compress */
 	if (prot->capa_cmd.compress &&
 	    strstr(str, prot->capa_cmd.compress) != NULL) {
-	    *capabilities |= CAPA_COMPRESS;
+	    *supports_compress = 1;
 	}
 	
 	/* check for auth */
 	if (prot->capa_cmd.auth &&
 	    (tmp = strstr(str, prot->capa_cmd.auth)) != NULL) {
-	    free(ret); /* avoid memory leak if duplicate mechlists */
 	    if (prot->capa_cmd.parse_mechlist)
 		ret = prot->capa_cmd.parse_mechlist(str, prot);
 	    else
@@ -1644,7 +1625,7 @@ static char *imap_parse_mechlist(const char *str, struct protocol_t *prot)
     char *tmp;
     int num = 0;
     
-    if (strstr(str, " SASL-IR")) {
+    if (strstr(str, "SASL-IR")) {
 	/* server supports initial response in AUTHENTICATE command */
 	prot->sasl_cmd.maxlen = USHRT_MAX;
     }
@@ -1659,7 +1640,7 @@ static char *imap_parse_mechlist(const char *str, struct protocol_t *prot)
 	strlcat(ret, tmp, strlen(ret) + (end - tmp) + 1);
 	
 	/* reset the string */
-	str = end;
+	str = end + 1;
     }
     
     return ret;
@@ -1707,27 +1688,20 @@ static int auth_imap(void)
 
 static int imap_do_auth(struct sasl_cmd_t *sasl_cmd,
 			void *rock __attribute__((unused)),
-			int login_enabled,
 			char *mech, char *mechlist)
 {
     int result = IMTEST_FAIL;
 
     if (mech) {
 	if (!strcasecmp(mech, "login")) {
-	    if (!login_enabled) {
-		printf("[Server advertised LOGINDISABLED]\n");
-	    } else {
-		result = auth_imap();
-	    }
-	} else if (!mechlist || !stristr(mechlist, mech)) {
-	    printf("[Server did not advertise AUTH=%s]\n", ucase(mech));
+	    result = auth_imap();
 	} else {
 	    result = auth_sasl(sasl_cmd, mech);
 	}
     } else {
 	if (mechlist) {
 	    result = auth_sasl(sasl_cmd, mechlist);
-	} else if (login_enabled) {
+	} else {
 	    result = auth_imap();
 	}
     }
@@ -1985,17 +1959,22 @@ static int auth_apop(char *apop_chal)
     unsigned int passlen;
     int i;
     MD5_CTX ctx;
-    unsigned char digest[MD5_DIGEST_LENGTH];
-    char digeststr[2*MD5_DIGEST_LENGTH+1];
+    unsigned char digest[16];
+    char digeststr[32];
     
+    if (!apop_chal) {
+	printf("[Server does not support APOP]\n");
+	return IMTEST_FAIL;
+    }
+
     interaction(SASL_CB_AUTHNAME, NULL, "Authname", &username, &userlen);
     interaction(SASL_CB_PASS,NULL, "Please enter your password",
 		&pass, &passlen);
     
-    MD5Init(&ctx);
-    MD5Update(&ctx,apop_chal,strlen(apop_chal));
-    MD5Update(&ctx,pass,passlen);
-    MD5Final(digest, &ctx);
+    MD5_Init(&ctx);
+    MD5_Update(&ctx,apop_chal,strlen(apop_chal));
+    MD5_Update(&ctx,pass,passlen);
+    MD5_Final(digest, &ctx);
     
     /* convert digest from binary to ASCII hex */
     for (i = 0; i < 16; i++)
@@ -2018,35 +1997,25 @@ static int auth_apop(char *apop_chal)
     }
 }
 
-static int pop3_do_auth(struct sasl_cmd_t *sasl_cmd, void *apop_chal,
-			int user_enabled, char *mech, char *mechlist)
+static int pop3_do_auth(struct sasl_cmd_t *sasl_cmd, void *rock,
+			char *mech, char *mechlist)
 {
     int result = IMTEST_FAIL;
     
     if (mech) {
 	if (!strcasecmp(mech, "apop")) {
-	    if (!apop_chal) {
-		printf("[Server did not advertise APOP challenge]\n");
-	    } else {
-		result = auth_apop((char *) apop_chal);
-	    }
+	    result = auth_apop((char *) rock);
 	} else if (!strcasecmp(mech, "user")) {
-	    if (!user_enabled) {
-		printf("[Server did not advertise USER]\n");
-	    } else {
-		result = auth_pop();
-	    }
-	} else if (!mechlist || !stristr(mechlist, mech)) {
-	    printf("[Server did not advertise SASL %s]\n", ucase(mech));
+	    result = auth_pop();
 	} else {
 	    result = auth_sasl(sasl_cmd, mech);
 	}
     } else {
 	if (mechlist) {
 	    result = auth_sasl(sasl_cmd, mechlist);
-	} else if (apop_chal) {
-	    result = auth_apop((char *) apop_chal);
-	} else if (user_enabled) {
+	} else if (rock) {
+	    result = auth_apop((char *) rock);
+	} else {
 	    result = auth_pop();
 	}
     }
@@ -2056,7 +2025,7 @@ static int pop3_do_auth(struct sasl_cmd_t *sasl_cmd, void *apop_chal,
 
 /********************************** NNTP *************************************/
 
-static int auth_nntp()
+static int auth_nntp(void)
 {
     char str[1024];
     /* we need username and password to do AUTHINFO USER/PASS */
@@ -2101,28 +2070,20 @@ static int auth_nntp()
 
 static int nntp_do_auth(struct sasl_cmd_t *sasl_cmd,
 			void *rock __attribute__((unused)),
-			int user_enabled, char *mech, char *mechlist)
+			char *mech, char *mechlist)
 {
     int result = IMTEST_OK;
 
     if (mech) {
 	if (!strcasecmp(mech, "user")) {
-	    if (!user_enabled) {
-		printf("[Server did not advertise AUTHINFO USER]\n");
-		result = IMTEST_FAIL;
-	    } else {
-		result = auth_nntp(user_enabled);
-	    }
-	} else if (!mechlist || !stristr(mechlist, mech)) {
-	    printf("[Server did not advertise SASL %s]\n", ucase(mech));
-	    result = IMTEST_FAIL;
+	    result = auth_nntp();
 	} else {
 	    result = auth_sasl(sasl_cmd, mech);
 	}
     } else {
 	if (mechlist) {
 	    result = auth_sasl(sasl_cmd, mechlist);
-	} else if (user_enabled) {
+	} else {
 	    result = auth_nntp();
 	}
     }
@@ -2147,7 +2108,6 @@ static char *nntp_parse_success(char *str)
 
 static int xmtp_do_auth(struct sasl_cmd_t *sasl_cmd,
 			void *rock __attribute__((unused)),
-			int login_enabled __attribute__((unused)),
 			char *mech, char *mechlist)
 {
     int result = IMTEST_OK;
@@ -2293,80 +2253,79 @@ void usage(char *prog, char *prot)
 
 
 static struct protocol_t protocols[] = {
-    { "imap", "imaps", "imap", 1,  /* LOGIN available until LOGINDISABLED */
+    { "imap", "imaps", "imap",
       { 1, NULL, NULL },
-      { "C01 CAPABILITY", "C01 ", " STARTTLS", " LOGINDISABLED", " AUTH=",
-	" COMPRESS=DEFLATE", &imap_parse_mechlist },
+      { "C01 CAPABILITY", "C01 ", " STARTTLS", " AUTH=", " COMPRESS=DEFLATE",
+	&imap_parse_mechlist },
       { "S01 STARTTLS", "S01 OK", "S01 NO", 0 },
-      { "A01 AUTHENTICATE", 0,  /* no init resp until SASL-IR advertised */
-	0, "A01 OK", "A01 NO", "+ ", "*", NULL, 0 },
+      { "A01 AUTHENTICATE", 0, 0, "A01 OK", "A01 NO", "+ ", "*", NULL, 0 },
       { "Z01 COMPRESS DEFLATE", "Z01 OK", "Z01 NO" },
       &imap_do_auth, { "Q01 LOGOUT", "Q01 " },
       &imap_init_conn, &generic_pipe, &imap_reset
     },
-    { "pop3", "pop3s", "pop", 0,   /* USER unavailable until advertised */
+    { "pop3", "pop3s", "pop",
       { 0, "+OK", &pop3_parse_banner },
-      { "CAPA", ".", "STLS", "USER", "SASL ", NULL, NULL },
+      { "CAPA", ".", "STLS", "SASL ", NULL, NULL },
       { "STLS", "+OK", "-ERR", 0 },
       { "AUTH", 255, 0, "+OK", "-ERR", "+ ", "*", NULL, 0 },
       { NULL, NULL, NULL, },
       &pop3_do_auth, { "QUIT", "+OK" }, NULL, NULL, NULL
     },
-    { "nntp", "nntps", "nntp", 0,  /* AUTHINFO USER unavail until advertised */
+    { "nntp", "nntps", "nntp",
       { 0, "20", NULL },
-      { "CAPABILITIES", ".", "STARTTLS", "AUTHINFO USER", "SASL ", NULL, NULL },
+      { "CAPABILITIES", ".", "STARTTLS", "SASL ", NULL, NULL },
       { "STARTTLS", "382", "580", 0 },
       { "AUTHINFO SASL", 512, 0, "28", "48", "383 ", "*",
 	&nntp_parse_success, 0 },
       { NULL, NULL, NULL, },
       &nntp_do_auth, { "QUIT", "205" }, NULL, NULL, NULL
     },
-    { "lmtp", NULL, "lmtp", 0,
+    { "lmtp", NULL, "lmtp",
       { 0, "220 ", NULL },
-      { "LHLO lmtptest", "250 ", "STARTTLS", NULL, "AUTH ", NULL, NULL },
+      { "LHLO lmtptest", "250 ", "STARTTLS", "AUTH ", NULL, NULL },
       { "STARTTLS", "220", "454", 0 },
       { "AUTH", 512, 0, "235", "5", "334 ", "*", NULL, 0 },
       { NULL, NULL, NULL, },
       &xmtp_do_auth, { "QUIT", "221" },
       &xmtp_init_conn, &generic_pipe, &xmtp_reset
     },
-    { "smtp", "smtps", "smtp", 0,
+    { "smtp", "smtps", "smtp",
       { 0, "220 ", NULL },
-      { "EHLO smtptest", "250 ", "STARTTLS", NULL, "AUTH ", NULL, NULL },
+      { "EHLO smtptest", "250 ", "STARTTLS", "AUTH ", NULL, NULL },
       { "STARTTLS", "220", "454", 0 },
       { "AUTH", 512, 0, "235", "5", "334 ", "*", NULL, 0 },
       { NULL, NULL, NULL, },
       &xmtp_do_auth, { "QUIT", "221" },
       &xmtp_init_conn, &generic_pipe, &xmtp_reset
     },
-    { "mupdate", NULL, "mupdate", 0,
+    { "mupdate", NULL, "mupdate",
       { 1, "* OK", NULL },
-      { NULL , "* OK", "* STARTTLS", NULL, "* AUTH ", "* COMPRESS \"DEFLATE\"", NULL },
+      { NULL , "* OK", "* STARTTLS", "* AUTH ", "* COMPRESS \"DEFLATE\"", NULL },
       { "S01 STARTTLS", "S01 OK", "S01 NO", 1 },
       { "A01 AUTHENTICATE", USHRT_MAX, 1, "A01 OK", "A01 NO", "", "*", NULL, 0 },
       { "Z01 COMPRESS \"DEFLATE\"", "Z01 OK", "Z01 NO" },
       NULL, { "Q01 LOGOUT", "Q01 " }, NULL, NULL, NULL
     },
-    { "sieve", NULL, SIEVE_SERVICE_NAME, 0,
+    { "sieve", NULL, SIEVE_SERVICE_NAME,
       { 1, "OK", NULL },
-      { "CAPABILITY", "OK", "\"STARTTLS\"", NULL, "\"SASL\" ", NULL, NULL },
+      { "CAPABILITY", "OK", "\"STARTTLS\"", "\"SASL\" ", NULL, NULL },
       { "STARTTLS", "OK", "NO", 1 },
       { "AUTHENTICATE", USHRT_MAX, 1, "OK", "NO", NULL, "*",
 	&sieve_parse_success, 1 },
       { NULL, NULL, NULL, },
       NULL, { "LOGOUT", "OK" }, NULL, NULL, NULL
     },
-    { "csync", NULL, "csync", 0,
+    { "csync", NULL, "csync",
       { 1, "* OK", NULL },
-      { NULL , "* OK", "* STARTTLS", NULL, "* SASL ", "* COMPRESS DEFLATE", NULL },
+      { NULL , "* OK", "* STARTTLS", "* SASL ", "* COMPRESS DEFLATE", NULL },
       { "STARTTLS", "OK", "NO", 1 },
       { "AUTHENTICATE", USHRT_MAX, 0, "OK", "NO", "+ ", "*", NULL, 0 },
       { "COMPRESS DEFLATE", "OK", "NO" },
       NULL, { "EXIT", "OK" }, NULL, NULL, NULL
     },
-    { NULL, NULL, NULL, 0,
+    { NULL, NULL, NULL,
       { 0, NULL, NULL },
-      { NULL, NULL, NULL, NULL, NULL, NULL, NULL },
+      { NULL, NULL, NULL, NULL, NULL, NULL },
       { NULL, NULL, NULL, 0 },
       { NULL, 0, 0, NULL, NULL, NULL, NULL, NULL, 0 },
       { NULL, NULL, NULL, },
@@ -2396,7 +2355,7 @@ int main(int argc, char **argv)
     char *port = "", *prot = "";
     int run_stress_test=0;
     int dotls=0, dossl=0, docompress=0;
-    unsigned long capabilities = 0;
+    int server_supports_tls, server_supports_compress;
     char str[1024];
     const char *pidfile = NULL;
     void *rock = NULL;
@@ -2650,8 +2609,9 @@ int main(int argc, char **argv)
 
 	if (protocol->banner.is_capa) {
 	    /* try to get the capabilities from the banner */
-	    mechlist = ask_capability(protocol, &capabilities, AUTO_BANNER);
-	    if (!mechlist && !(capabilities & CAPA_STARTTLS)) {
+	    mechlist = ask_capability(protocol, &server_supports_tls,
+				      &server_supports_compress, AUTO_BANNER);
+	    if (!mechlist && !server_supports_tls) {
 		/* found no capabilities in banner -> get them explicitly */
 		protocol->banner.is_capa = 0;
 	    }
@@ -2670,11 +2630,12 @@ int main(int argc, char **argv)
 				 strlen(protocol->banner.resp)));
 	}	
 	if (!protocol->banner.is_capa) {
-	    mechlist = ask_capability(protocol, &capabilities, AUTO_NO);
+	    mechlist = ask_capability(protocol, &server_supports_tls,
+				      &server_supports_compress, AUTO_NO);
 	}
 	
 #ifdef HAVE_SSL
-	if ((dossl==0) && (dotls==1) && (capabilities & CAPA_STARTTLS)) {
+	if ((dossl==0) && (dotls==1) && (server_supports_tls==1)) {
 	    char *resp;
 
 	    printf("C: %s\r\n", protocol->tls_cmd.cmd);
@@ -2693,17 +2654,18 @@ int main(int argc, char **argv)
 		    printf("Asking for capabilities again "
 			   "since they might have changed\n");
 		if (mechlist) free(mechlist);
-		mechlist = ask_capability(protocol, &capabilities,
+		mechlist = ask_capability(protocol, &server_supports_tls,
+					  &server_supports_compress,
 					  protocol->tls_cmd.auto_capa);
 	    }
 	    
-	} else if ((dotls==1) && !(capabilities & CAPA_STARTTLS)) {
+	} else if ((dotls==1) && (server_supports_tls!=1)) {
 	    imtest_fatal("STARTTLS not supported by the server!\n");
 	}
 #endif /* HAVE_SSL */
 
 #ifdef HAVE_ZLIB
-	if ((reauth == 1) && (docompress==1) && (capabilities & CAPA_COMPRESS)) {
+	if ((reauth == 1) && (docompress==1) && (server_supports_compress==1)) {
 	char *resp;
 
 	printf("C: %s\r\n", protocol->compress_cmd.cmd);
@@ -2727,7 +2689,6 @@ int main(int argc, char **argv)
 
 	if (protocol->do_auth)
 	    result = protocol->do_auth(&protocol->sasl_cmd, rock,
-				       capabilities & CAPA_LOGIN,
 				       mechanism, mechlist);
 	else {
 	    if (mechanism) {
@@ -2750,13 +2711,7 @@ int main(int argc, char **argv)
 	} else {
 	    const char *s = sasl_errstring(result, NULL, NULL);
 	    
-	    fprintf(stderr, "Authentication failed. %s\n", s);
-
-	    if (output_socket) {
-		/* send LOGOUT and exit */
-		logout(&protocol->logout_cmd, 0);
-		exit(1);
-	    }
+	    printf("Authentication failed. %s\n", s);
 	}
 	
 	result = sasl_getprop(conn, SASL_SSF, &ssfp);
@@ -2791,7 +2746,8 @@ int main(int argc, char **argv)
 		    }
 		    prot_BLOCK(pin);
 		}
-		new_mechlist = ask_capability(protocol, &capabilities,
+		new_mechlist = ask_capability(protocol, &server_supports_tls,
+					      &server_supports_compress,
 					      protocol->sasl_cmd.auto_capa);
 		if (new_mechlist && strcmp(new_mechlist, mechlist)) {
 		    printf("WARNING: possible MITM attack: "
