@@ -38,6 +38,8 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * $Id: prot.c,v 1.100 2010/06/28 12:06:43 brong Exp $
  */
 
 #include <config.h>
@@ -84,7 +86,9 @@ struct protgroup
  * Create a new protection stream for file descriptor 'fd'.  Stream
  * will be used for writing iff 'write' is nonzero.
  */
-struct protstream *prot_new(int fd, int write)
+struct protstream *prot_new(fd, write)
+int fd;
+int write;
 {
     struct protstream *newstream;
 
@@ -101,23 +105,9 @@ struct protstream *prot_new(int fd, int write)
     if(write)
 	newstream->cnt = PROT_BUFSIZE;
 
-    return newstream;
-}
-
-/* Create a protstream which is just an interface to a mapped piece of
- * memory, allowing prot commands to be used to read from it */
-struct protstream *prot_readmap(const char *buf, uint32_t len)
-{
-    struct protstream *newstream;
-
-    newstream = (struct protstream *) xzmalloc(sizeof(struct protstream));
-    /* dodgy, but the alternative is two pointers */
-    newstream->ptr = (unsigned char *)buf;
-    newstream->cnt = len;
-    newstream->fixedsize = 1;
-    newstream->fd = PROT_NO_FD;
-    newstream->logfd = PROT_NO_FD;
-    newstream->big_buffer = PROT_NO_FD;
+    newstream->can_unget = 0;
+    newstream->bytes_in = 0;
+    newstream->bytes_out = 0;
 
     return newstream;
 }
@@ -144,7 +134,7 @@ int prot_free(struct protstream *s)
     if (s->zbuf) free(s->zbuf);
 #endif
 
-    free(s);
+    free((char*)s);
 
     return 0;
 }
@@ -158,12 +148,6 @@ int prot_setlog(struct protstream *s, int fd)
     return 0;
 }
 
-int prot_setisclient(struct protstream *s, int val)
-{
-    s->isclient = val;
-    return 0;
-}
-
 #ifdef HAVE_SSL
 
 /*
@@ -226,7 +210,9 @@ int prot_sasldecode(struct protstream *s, int n)
  * Turn on SASL for this connection
  */
 
-int prot_setsasl(struct protstream *s, sasl_conn_t *conn)
+int prot_setsasl(s, conn)
+struct protstream *s;
+sasl_conn_t *conn;
 {
     const void *ssfp;
     int result;
@@ -272,17 +258,6 @@ int prot_setsasl(struct protstream *s, sasl_conn_t *conn)
     return 0;
 }
 
-/*
- * Turn off SASL for this connection
- */
-
-void prot_unsetsasl(struct protstream *s)
-{
-    s->conn = NULL;
-    s->maxplain = PROT_BUFSIZE;
-    s->saslssf = 0;
-}
-
 #ifdef HAVE_ZLIB
 
 #define ZLARGE_DIFF_CHUNK (5120) /* 5K */
@@ -324,12 +299,12 @@ int prot_setcompress(struct protstream *s)
 
 	s->zlevel = Z_DEFAULT_COMPRESSION;
 	zr = deflateInit2(zstrm, s->zlevel, Z_DEFLATED,
-		          -MAX_WBITS, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
+		          -15, 8, Z_DEFAULT_STRATEGY);
     }
     else {
 	zstrm->next_in = Z_NULL;
 	zstrm->avail_in = 0;
-	zr = inflateInit2(zstrm, -MAX_WBITS);
+	zr = inflateInit2(zstrm, -15);
     }
 
     if (zr != Z_OK)
@@ -574,7 +549,6 @@ int prot_fill(struct protstream *s)
     /* Zero errno just in case */
     errno = 0;
 
-    if (s->fixedsize) s->eof = 1;
     if (s->eof || s->error) return EOF;
 
     do {
@@ -766,30 +740,10 @@ int prot_fill(struct protstream *s)
 }
 
 /*
- * If 's' is an input stream, discard any pending/buffered data.  Otherwise,
  * Write out any buffered data in the stream 's'
  */
 int prot_flush(struct protstream *s) 
 {
-    if (!s->write) {
-	int c, save_dontblock = s->dontblock;
-
-	/* Set stream to nonblocking mode */
-	if (!save_dontblock) nonblock(s->fd, (s->dontblock = 1));
-
-	/* Ingest any pending input */
-	while ((c = prot_fill(s)) != EOF);
-
-	/* Reset stream to previous blocking mode */
-	if (!save_dontblock) nonblock(s->fd, (s->dontblock = 0));
-
-	/* Discard any buffered input */
-	s->cnt = 0;
-	s->can_unget = 0;
-
-	return 0;
-    }
-
     return prot_flush_internal(s, 1);
 }
 
@@ -1273,9 +1227,6 @@ int prot_printf(struct protstream *s, const char *fmt, ...)
 	    prot_write(s, buf, strlen(buf));
 	    break;
 
-	/* according to linux 'man 3 printf' this is supposed to be
-	 * of type "ptrdiff_t", not "size_t" - but don't want to break
-	 * running code! */
 	case 't': {
 	    size_t tu;
 	    ssize_t td;
@@ -1309,29 +1260,6 @@ int prot_printf(struct protstream *s, const char *fmt, ...)
 	    (void)prot_putc(i, s);
 	    break;
 
-	case 'z': {
-	    size_t zu;
-	    ssize_t zd;
-
-	    switch (*++percent) {
-	    case 'u':
-		zu = va_arg(pvar, size_t);
-		snprintf(buf, sizeof(buf), "%zu", zu);
-		prot_write(s, buf, strlen(buf));
-		break;
-
-	    case 'd':
-		zd = va_arg(pvar, ssize_t);
-		snprintf(buf, sizeof(buf), "%zd", zd);
-		prot_write(s, buf, strlen(buf));
-		break;
-
-	    default:
-		abort();
-	    }
-	    break;
-	}
-
 	default:
 	    abort();
 	}
@@ -1346,10 +1274,7 @@ int prot_printf(struct protstream *s, const char *fmt, ...)
 int prot_printliteral(struct protstream *out, const char *s, size_t size)
 {
     int r;
-    if (out->isclient)
-	r = prot_printf(out, "{" SIZE_T_FMT "+}\r\n", size);
-    else
-	r = prot_printf(out, "{" SIZE_T_FMT "}\r\n", size);
+    r = prot_printf(out, "{" SIZE_T_FMT "+}\r\n", size);
     if (r) return r;
     return prot_write(out, s, size);
 }
@@ -1727,17 +1652,13 @@ int prot_ungetc(int c, struct protstream *s)
 {
     assert(!s->write);
 
-    if (c == EOF) return EOF;
-
     if (!s->can_unget)
 	fatal("Can't unwind any more", EC_SOFTWARE);
 
     s->cnt++;
     s->can_unget--;
     s->bytes_in--;
-    s->ptr--;
-    if (*s->ptr != c)
-	fatal("Trying to unput wrong character", EC_SOFTWARE);
+    *--(s->ptr) = c;
 
     return c;
 }
