@@ -50,6 +50,7 @@
 #include "prot.h"
 #include "mystring.h"
 
+#define LEXER_STATE_TAG         60
 #define LEXER_STATE_RECOVER     61
 #define LEXER_STATE_RECOVER_CR  62
 #define LEXER_STATE_CR          63
@@ -79,8 +80,6 @@
 #define CAPABILITY   407
 #define HAVESPACE    408
 #define STARTTLS     409
-#define NOOP         410
-#define UNAUTHENTICATE 411
 
 
 int lex_init(void);
